@@ -1,4 +1,6 @@
 const Gio = imports.gi.Gio;
+const GObject = imports.gi.GObject;
+const Polari = imports.gi.Polari;
 const Signals = imports.signals;
 const Tp = imports.gi.TelepathyGLib;
 
@@ -14,16 +16,19 @@ var AccountsMonitor = class {
         this._accountSettings = new Map();
 
         this._app = Gio.Application.get_default();
-        this._app.connect('prepare-shutdown',
-                          this._onPrepareShutdown.bind(this));
 
-        this._accountManager = Tp.AccountManager.dup();
+        if (!this._app.isTestInstance)
+            this._app.connect('prepare-shutdown',
+                              this._onPrepareShutdown.bind(this));
 
-        let factory = this._accountManager.get_factory();
+        let factory = new ClientFactory();
         factory.add_channel_features([Tp.Channel.get_feature_quark_group()]);
         factory.add_channel_features([Tp.Channel.get_feature_quark_contacts()]);
         factory.add_contact_features([Tp.ContactFeature.ALIAS]);
 
+        this._accountManager = Tp.AccountManager.new_with_factory(factory);
+        this._accountManager.set_default();
+
         this._preparedCallbacks = [];
         this._accountManager.prepare_async(null, this._onPrepared.bind(this));
     }
@@ -32,8 +37,8 @@ var AccountsMonitor = class {
         return [...this._accounts.values()];
     }
 
-    get enabledAccounts() {
-        return [...this._accounts.values()].filter(a => a.enabled);
+    get visibleAccounts() {
+        return [...this._accounts.values()].filter(a => a.visible);
     }
 
     get accountManager() {
@@ -118,6 +123,12 @@ var AccountsMonitor = class {
             account.connect('notify::connection-status', () => {
                 this.emit('account-status-changed', account);
             });
+        account._visibleNotifyId =
+            account.connect('notify::visible', () => {
+                this.emit(account.visible ? 'account-shown'
+                                          : 'account-hidden', account);
+                this.emit('accounts-changed');
+            });
         this._accounts.set(account.object_path, account);
 
         this.emit('account-added', account);
@@ -131,6 +142,9 @@ var AccountsMonitor = class {
         account.disconnect(account._statusNotifyId);
         delete account._statusNotifyId;
 
+        account.disconnect(account._visibleNotifyId);
+        delete account._visibleNotifyId;
+
         this.emit('account-removed', account);
         this.emit('accounts-changed');
     }
@@ -144,3 +158,42 @@ var AccountsMonitor = class {
     }
 };
 Signals.addSignalMethods(AccountsMonitor.prototype);
+
+const ClientFactory = GObject.registerClass(
+class ClientFactory extends Polari.ClientFactory {
+    vfunc_create_account(objectPath) {
+        return new PolariAccount({ factory: this,
+                                   dbus_daemon: this.dbus_daemon,
+                                   bus_name: Tp.ACCOUNT_MANAGER_BUS_NAME,
+                                   object_path: objectPath });
+    }
+});
+
+const PolariAccount = GObject.registerClass({
+    Properties: {
+        visible: GObject.ParamSpec.boolean('visible',
+                                           'visible',
+                                           'visible',
+                                           GObject.ParamFlags.READWRITE |
+                                           GObject.ParamFlags.EXPLICIT_NOTIFY,
+                                           true)
+    }
+}, class PolariAccount extends Tp.Account {
+    _init(params) {
+        this._visible = true;
+
+        super._init(params);
+    }
+
+    get visible() {
+        return this._visible;
+    }
+
+    set visible(value) {
+        if (this._visible == value)
+            return;
+
+        this._visible = value;
+        this.notify('visible');
+    }
+});
