@@ -1,43 +1,39 @@
+const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
-const Pango = imports.gi.Pango;
-const Tp = imports.gi.TelepathyGLib;
-
-const Lang = imports.lang;
 const Mainloop = imports.mainloop;
+const Pango = imports.gi.Pango;
 
 const TIMEOUT = 7;
 const COMMAND_OUTPUT_REVEAL_TIME = 3;
 
-var AppNotification = new Lang.Class({
-    Name: 'AppNotification',
-    Abstract: true,
-    Extends: Gtk.Revealer,
+var AppNotification = GObject.registerClass(
+class AppNotification extends Gtk.Revealer {
+    _init() {
+        if (this.constructor.name == 'AppNotification')
+            throw new Error('Cannot instantiate abstract class AppNotification');
 
-    _init: function() {
-        this.parent({ reveal_child: true,
+        super._init({ reveal_child: true,
                       transition_type: Gtk.RevealerTransitionType.SLIDE_DOWN });
         this.connect('notify::child-revealed',
-                     Lang.bind(this, this._onChildRevealed));
-    },
+                     this._onChildRevealed.bind(this));
+    }
 
-    close: function() {
+    close() {
         this.reveal_child = false;
-    },
+    }
 
-    _onChildRevealed: function() {
+    _onChildRevealed() {
         if (!this.child_revealed)
             this.destroy();
     }
 });
 
-var MessageNotification = new Lang.Class({
-    Name: 'MessageNotification',
-    Extends: AppNotification,
+var MessageNotification = GObject.registerClass(
+class MessageNotification extends AppNotification {
+    _init(label, iconName) {
+        super._init();
 
-    _init: function(label, iconName) {
-        this.parent();
-
-        Mainloop.timeout_add_seconds(TIMEOUT, Lang.bind(this, this.close));
+        Mainloop.timeout_add_seconds(TIMEOUT, this.close.bind(this));
 
         this._box = new Gtk.Box({ spacing: 12 });
 
@@ -49,15 +45,15 @@ var MessageNotification = new Lang.Class({
 
         let closeButton = new Gtk.Button({ relief: Gtk.ReliefStyle.NONE });
         closeButton.image = new Gtk.Image({ icon_name: 'window-close-symbolic' });
-        closeButton.connect('clicked', Lang.bind(this, this.close));
+        closeButton.connect('clicked', this.close.bind(this));
         this._box.pack_end(closeButton, false, false, 0);
 
         this.add(this._box);
         this.show_all();
-    },
+    }
 
 
-    addButton: function(label, callback) {
+    addButton(label, callback) {
         let button = new Gtk.Button({ label: label, visible: true });
         button.connect('clicked', () => {
             if (callback)
@@ -69,13 +65,11 @@ var MessageNotification = new Lang.Class({
     }
 });
 
-var UndoNotification = new Lang.Class({
-    Name: 'UndoNotification',
-    Extends: MessageNotification,
-    Signals: { closed: {}, undo: {} },
-
-    _init: function(label) {
-        this.parent(label);
+var UndoNotification = GObject.registerClass({
+    Signals: { closed: {}, undo: {} }
+}, class UndoNotification extends MessageNotification {
+    _init(label) {
+        super._init(label);
 
         this._undo = false;
         this._closed = false;
@@ -83,38 +77,36 @@ var UndoNotification = new Lang.Class({
         this.connect('destroy', () => { this.close(); });
 
         this.addButton(_("Undo"), () => { this._undo = true; });
-    },
+    }
 
-    close: function() {
+    close() {
         if (this._closed)
             return;
 
         this._closed = true;
         this.emit(this._undo ? 'undo' : 'closed');
-        this.parent();
+        super.close();
     }
 });
 
-var CommandOutputNotification = new Lang.Class({
-    Name: 'CommandOutputNotification',
-    Extends: AppNotification,
-    Abstract: true,
+var CommandOutputNotification = GObject.registerClass(
+class CommandOutputNotification extends AppNotification {
+    _init() {
+        if (this.constructor.name == 'CommandOutputNotification')
+            throw new Error('Cannot instantiate abstract class CommandOutputNotification');
 
-    _init: function() {
-        this.parent();
+        super._init();
 
         this.transition_type = Gtk.RevealerTransitionType.SLIDE_UP;
         Mainloop.timeout_add_seconds(COMMAND_OUTPUT_REVEAL_TIME,
-                                     Lang.bind(this, this.close));
+                                     this.close.bind(this));
     }
 });
 
-var SimpleOutput = new Lang.Class({
-    Name: 'SimpleOutput',
-    Extends: CommandOutputNotification,
-
-    _init: function(text) {
-        this.parent();
+var SimpleOutput = GObject.registerClass(
+class SimpleOutput extends CommandOutputNotification {
+    _init(text) {
+        super._init();
 
         let label = new Gtk.Label({ label: text,
                                     vexpand: true,
@@ -125,12 +117,10 @@ var SimpleOutput = new Lang.Class({
     }
 });
 
-var GridOutput = new Lang.Class({
-    Name: 'GridOutput',
-    Extends: CommandOutputNotification,
-
-    _init: function(header, items) {
-        this.parent();
+var GridOutput = GObject.registerClass(
+class GridOutput extends CommandOutputNotification {
+    _init(header, items) {
+        super._init();
 
         let numItems = items.length;
         let numCols = Math.min(numItems, 4);
@@ -157,12 +147,10 @@ var GridOutput = new Lang.Class({
     }
 });
 
-var NotificationQueue = new Lang.Class({
-    Name: 'NotificationQueue',
-    Extends: Gtk.Frame,
-
-    _init: function() {
-        this.parent({ valign: Gtk.Align.START,
+var NotificationQueue = GObject.registerClass(
+class NotificationQueue extends Gtk.Frame {
+    _init() {
+        super._init({ valign: Gtk.Align.START,
                       halign: Gtk.Align.CENTER,
                       margin_start: 24, margin_end: 24,
                       no_show_all: true });
@@ -171,27 +159,25 @@ var NotificationQueue = new Lang.Class({
         this._grid = new Gtk.Grid({ orientation: Gtk.Orientation.VERTICAL,
                                     row_spacing: 6, visible: true });
         this.add(this._grid);
-    },
+    }
 
-    addNotification: function(notification) {
+    addNotification(notification) {
         this._grid.add(notification);
 
-        notification.connect('destroy', Lang.bind(this, this._onChildDestroy));
+        notification.connect('destroy', this._onChildDestroy.bind(this));
         this.show();
-    },
+    }
 
-    _onChildDestroy: function() {
+    _onChildDestroy() {
         if (this._grid.get_children().length == 0)
            this.hide();
     }
 });
 
-var CommandOutputQueue = new Lang.Class({
-    Name: 'CommandOutputQueue',
-    Extends: NotificationQueue,
-
-    _init: function() {
-        this.parent();
+var CommandOutputQueue = GObject.registerClass(
+class CommandOutputQueue extends NotificationQueue {
+    _init() {
+        super._init();
 
         this.valign = Gtk.Align.END;
         this.get_style_context().add_class('irc-feedback');
