@@ -45,17 +45,24 @@ var Application = GObject.registerClass({
         this.add_main_option('start-client', 0,
                              GLib.OptionFlags.NONE, GLib.OptionArg.NONE,
                              _("Start Telepathy client"), null);
+        this.add_main_option('test-instance', 0,
+                             GLib.OptionFlags.NONE, GLib.OptionArg.NONE,
+                             _("Allow running alongside another instance"), null);
         this.add_main_option('version', 0,
                              GLib.OptionFlags.NONE, GLib.OptionArg.NONE,
                              _("Print version and exit"), null);
         this.connect('handle-local-options', (o, dict) => {
+            let v = dict.lookup_value('test-instance', null);
+            if (v && v.get_boolean())
+                this._maybeMakeNonUnique();
+
             try {
                 this.register(null);
             } catch(e) {
                 return 1;
             }
 
-            let v = dict.lookup_value('start-client', null);
+            v = dict.lookup_value('start-client', null);
             if (v && v.get_boolean()) {
                 this.activate_action('start-client', null);
                 return 0;
@@ -116,6 +123,28 @@ var Application = GObject.registerClass({
         }
     }
 
+    _maybeMakeNonUnique() {
+        let bus = Gio.BusType.SESSION;
+        let name = this.application_id;
+        let flags = Gio.BusNameWatcherFlags.NONE;
+
+        let handled = false;
+        let id = Gio.bus_watch_name(bus, name, flags, () => {
+                debug('Running as test instance alongside primary instance');
+                this.set_flags(this.flags | Gio.ApplicationFlags.NON_UNIQUE);
+                handled = true;
+        }, () => {
+            debug('No primary instance found, running normally');
+            handled = true;
+        });
+
+        // Evil-ish ...
+        let main = GLib.MainContext.default();
+        while (!handled)
+            main.iteration(true);
+        Gio.bus_unwatch_name(id);
+    }
+
     vfunc_dbus_register(conn, path) {
         if (!Utils.isFlatpakSandbox())
             return true;
@@ -169,6 +198,9 @@ var Application = GObject.registerClass({
           { name: 'connect-account',
             activate: this._onConnectAccount.bind(this),
             parameter_type: GLib.VariantType.new('o') },
+          { name: 'disconnect-account',
+            activate: this._onConnectAccount.bind(this),
+            parameter_type: GLib.VariantType.new('o') },
           { name: 'reconnect-account',
             activate: this._onConnectAccount.bind(this),
             parameter_type: GLib.VariantType.new('o') },
@@ -250,7 +282,9 @@ var Application = GObject.registerClass({
         this._accountsMonitor = AccountsMonitor.getDefault();
         this._userStatusMonitor = UserStatusMonitor.getDefault();
         this._networksManager = NetworksManager.getDefault();
-        this._serverRoomManager = ServerRoomManager.getDefault();
+
+        if (!Gio.NetworkMonitor.get_default().get_network_metered())
+            this._serverRoomManager = ServerRoomManager.getDefault();
 
         this._accountsMonitor.connect('account-status-changed',
                                       this._onAccountStatusChanged.bind(this));
@@ -343,7 +377,7 @@ var Application = GObject.registerClass({
     _openURIs(uris, time) {
         let map = {};
 
-        this._accountsMonitor.enabledAccounts.forEach(a => {
+        this._accountsMonitor.visibleAccounts.forEach(a => {
             let params = a.dup_parameters_vardict().deep_unpack();
             map[a.get_object_path()] = {
                 server: params.server.deep_unpack(),
@@ -457,6 +491,10 @@ var Application = GObject.registerClass({
         return savedRooms.n_children() == 0;
     }
 
+    get isTestInstance() {
+        return this.flags & Gio.ApplicationFlags.NON_UNIQUE;
+    }
+
     _updateUserListAction() {
         let room = this.active_window.active_room;
         let action = this.lookup_action('user-list');
@@ -656,10 +694,14 @@ var Application = GObject.registerClass({
         action.change_state(GLib.Variant.new('b', !state.get_boolean()));
     }
 
-    _onRemoveConnection(action, parameter){
+    _onRemoveConnection(action, parameter) {
         let accountPath = parameter.deep_unpack();
         let account = this._accountsMonitor.lookupAccount(accountPath);
-        account.set_enabled_async(false, () => {
+
+        account.set_enabled_async(false, (o, res) => {
+            account.set_enabled_finish(res);
+            account.visible = false;
+
             let label = _("%s removed.").format(account.display_name);
             let n = new AppNotifications.UndoNotification(label);
             this.notificationQueue.addNotification(n);
@@ -670,8 +712,9 @@ var Application = GObject.registerClass({
                 });
             });
             n.connect('undo', () => {
-                account.set_enabled_async(true, (a, res) => {
-                    a.set_enabled_finish(res); // TODO: Check for errors
+                account.set_enabled_async(true, (o, res) => {
+                  account.set_enabled_finish(res);
+                  account.visible = true;
                 });
             });
         });
@@ -724,7 +767,7 @@ var Application = GObject.registerClass({
         let params = {
             name: 'Polari',
             account_manager: this._accountsMonitor.accountManager,
-            uniquify_name: false
+            uniquify_name: this.isTestInstance
         };
         this._telepathyClient = new TelepathyClient(params);
     }
@@ -753,7 +796,8 @@ var Application = GObject.registerClass({
                 'Jonas Danielsson <jonas@threetimestwo.org>',
                 'Rares Visalom <rares.visalom@gmail.com>',
                 'Danny Mølgaard <moelgaard.dmp@gmail.com>',
-                'Justyn Temme <Justyntemme@gmail.com>'
+                'Justyn Temme <Justyntemme@gmail.com>',
+                'unkemptArc99 <abhishekbhardwaj540@gmail.com>'
             ],
             artists: [
                 'Sam Hewitt <hewittsamuel@gmail.com>',
