@@ -148,24 +148,14 @@ var ButtonTag = GObject.registerClass({
                                            false)
     },
     Signals: {
-        'button-press-event': {
-            flags: GObject.SignalFlags.RUN_LAST,
-            param_types: [Gdk.Event.$gtype],
-            return_type: GObject.TYPE_BOOLEAN,
-            accumulator: GObject.AccumulatorType.TRUE_HANDLED
-        },
-        'button-release-event': {
-            flags: GObject.SignalFlags.RUN_LAST,
-            param_types: [Gdk.Event.$gtype],
-            return_type: GObject.TYPE_BOOLEAN,
-            accumulator: GObject.AccumulatorType.TRUE_HANDLED
-        },
-        'clicked': { }
+        'clicked': { },
+        'popup-menu': { }
     },
 }, class ButtonTag extends Gtk.TextTag {
     _init(params) {
         this._hover = false;
-        this._pressed = false;
+
+        this._gesture = null;
 
         super._init(params);
     }
@@ -182,41 +172,40 @@ var ButtonTag = GObject.registerClass({
         this.notify('hover');
     }
 
-    on_notify(pspec) {
-        if (pspec.name == 'hover' && !this.hover)
-            this._pressed = false;
-    }
-
-    'on_button-press-event'(event) {
-        let [, button] = event.get_button();
-        this._pressed = button == Gdk.BUTTON_PRIMARY;
-
-        return Gdk.EVENT_STOP;
-    }
-
-    'on_button-release-event'(event) {
-        let [, button] = event.get_button();
-        if (!(button == Gdk.BUTTON_PRIMARY && this._pressed))
-            return Gdk.EVENT_PROPAGATE;
-
-        this._pressed = false;
-        this.emit('clicked');
-        return Gdk.EVENT_STOP;
-    }
-
     vfunc_event(object, event, iter) {
-        let type = event.get_event_type();
+        this._ensureGesture(object);
 
-        if (!this._hover)
-            return Gdk.EVENT_PROPAGATE;
+        if (this._gesture.handle_event(event) ||
+            this._gesture.is_recognized())
+            return Gdk.EVENT_STOP;
 
-        if (type != Gdk.EventType.BUTTON_PRESS &&
-            type != Gdk.EventType.BUTTON_RELEASE)
-            return Gdk.EVENT_PROPAGATE;
+        return Gdk.EVENT_PROPAGATE;
+    }
 
-        let isPress = type == Gdk.EventType.BUTTON_PRESS;
-        return this.emit(isPress ? 'button-press-event'
-                                 : 'button-release-event', event);
+    _ensureGesture(widget) {
+        if (this._gesture)
+            return;
+
+        this._gesture = new Gtk.GestureMultiPress({ widget,
+                                                    button: 0,
+                                                    exclusive: true });
+
+        this._gesture.connect('pressed', (gesture, nPress) => {
+            if (!this._hover || nPress > 1)
+                return;
+
+            let button = this._gesture.get_current_button();
+            if (button == Gdk.BUTTON_SECONDARY)
+                this.emit('popup-menu');
+        });
+        this._gesture.connect('released', (gesture, nPress) => {
+            if (!this._hover || nPress > 1)
+                return;
+
+            let button = this._gesture.get_current_button();
+            if (button == Gdk.BUTTON_PRIMARY)
+                this.emit('clicked');
+        });
     }
 });
 
@@ -375,8 +364,9 @@ var ChatView = GObject.registerClass({
         this._app = Gio.Application.get_default();
         DropTargetIface.addTargets(this, this._view);
 
-        this._app.connect('room-focus-changed',
-                          this._checkMessages.bind(this));
+        this._roomFocusChangedId =
+            this._app.connect('room-focus-changed',
+                              this._checkMessages.bind(this));
 
         this._hoverCursor = Gdk.Cursor.new_from_name(this.get_display(),
                                                      'pointer');
@@ -409,11 +399,6 @@ var ChatView = GObject.registerClass({
         this._nickStatusChangedId = this._userTracker.watchRoomStatus(this._room,
                                     null,
                                     this._onNickStatusChanged.bind(this));
-
-        this.connect('destroy', () => {
-            this._userTracker.unwatchRoomStatus(this._room, this._nickStatusChangedId);
-            this._userTracker = null;
-        });
     }
 
     _createTags() {
@@ -526,10 +511,20 @@ var ChatView = GObject.registerClass({
             this._room.disconnect(this._roomSignals[i]);
         this._roomSignals = [];
 
+        if (this._roomFocusChangedId)
+            this._app.disconnect(this._roomFocusChangedId);
+        this._roomFocusChangedId = 0;
+
         if (this._backlogTimeoutId)
             Mainloop.source_remove(this._backlogTimeoutId);
         this._backlogTimeoutId = 0;
 
+        if (this._nickStatusChangedId)
+            this._userTracker.unwatchRoomStatus(this._room,
+                                                this._nickStatusChangedId);
+        this._nickStatusChangedId = 0;
+        this._userTracker = null;
+
         this._logWalker.run_dispose();
         this._logWalker = null;
     }
@@ -761,7 +756,7 @@ var ChatView = GObject.registerClass({
         this._pending.delete(id);
     }
 
-    _showUrlContextMenu(url, button, time) {
+    _showUrlContextMenu(url) {
         let menu = new Gtk.Menu();
 
         let item = new Gtk.MenuItem({ label: _("Open Link") });
@@ -778,7 +773,7 @@ var ChatView = GObject.registerClass({
         menu.append(item);
 
         menu.show_all();
-        menu.popup(null, null, null, button, time);
+        menu.popup_at_pointer(null);
     }
 
     _handleButtonTagsHover(view, event) {
@@ -1376,13 +1371,8 @@ var ChatView = GObject.registerClass({
         tag.connect('clicked', () => {
             Utils.openURL(url, Gtk.get_current_event_time());
         });
-        tag.connect('button-press-event', (tag, event) => {
-            let [, button] = event.get_button();
-            if (button != Gdk.BUTTON_SECONDARY)
-                return Gdk.EVENT_PROPAGATE;
-
-            this._showUrlContextMenu(url, button, event.get_time());
-            return Gdk.EVENT_STOP;
+        tag.connect('popup-menu', () => {
+            this._showUrlContextMenu(url);
         });
         return tag;
     }
