@@ -112,17 +112,26 @@ var ConnectionsList = GObject.registerClass({
         this._list.set_placeholder(placeholder);
 
         this._accountsMonitor = AccountsMonitor.getDefault();
-        this._accountsMonitor.connect('account-added', (mon, account) => {
-            this._setAccountRowSensitive(account, false);
-        });
-        this._accountsMonitor.connect('account-removed', (mon, account) => {
-            this._setAccountRowSensitive(account, true);
-        });
+        let accountAddedId =
+            this._accountsMonitor.connect('account-added', (mon, account) => {
+                this._setAccountRowSensitive(account, false);
+            });
+        let accountRemovedId =
+            this._accountsMonitor.connect('account-removed', (mon, account) => {
+                this._setAccountRowSensitive(account, true);
+            });
 
         this._networksManager = NetworksManager.getDefault();
-        this._networksManager.connect('changed',
-                                      this._networksChanged.bind(this));
+        let networksChangedId =
+            this._networksManager.connect('changed',
+                                          this._networksChanged.bind(this));
         this._networksChanged();
+
+        this.connect('destroy', () => {
+            this._accountsMonitor.disconnect(accountAddedId);
+            this._accountsMonitor.disconnect(accountRemovedId);
+            this._networksManager.disconnect(networksChangedId);
+        });
     }
 
     get favorites_only() {
@@ -262,7 +271,7 @@ var ConnectionDetails = GObject.registerClass({
 }, class ConnectionDetails extends Gtk.Grid {
     _init(params) {
         this._networksManager = NetworksManager.getDefault();
-        this._networksManager.connect('changed', () => {
+        let id = this._networksManager.connect('changed', () => {
             this.notify('has-service');
         });
 
@@ -270,6 +279,10 @@ var ConnectionDetails = GObject.registerClass({
 
         super._init(params);
 
+        this.connect('destroy', () => {
+            this._networksManager.disconnect(id);
+        });
+
         this._nameEntry.connect('changed',
                                 this._onCanConfirmChanged.bind(this));
         this._serverEntry.connect('changed',
@@ -498,9 +511,11 @@ var ConnectionProperties = GObject.registerClass({
         });
         this.set_default_response(Gtk.ResponseType.OK);
 
-        account.connect('notify::connection-status',
-                        this._syncErrorMessage.bind(this));
+        let id = account.connect('notify::connection-status',
+                                 this._syncErrorMessage.bind(this));
         this._syncErrorMessage(account);
+
+        this.connect('destroy', () => { account.disconnect(id); });
     }
 
     _syncErrorMessage(account) {
