@@ -5,73 +5,74 @@ const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
 const Gspell = imports.gi.Gspell;
 const Gtk = imports.gi.Gtk;
+const Mainloop = imports.mainloop;
+const Tp = imports.gi.TelepathyGLib;
 
 const ChatView = imports.chatView;
-const EmojiPicker = imports.emojiPicker;
-const IrcParser = imports.ircParser;
-const Lang = imports.lang;
-const Mainloop = imports.mainloop;
-const TabCompletion = imports.tabCompletion;
-const Tp = imports.gi.TelepathyGLib;
-const PasteManager = imports.pasteManager;
+const {DropTargetIface} = imports.pasteManager;
+const {EmojiPicker} = imports.emojiPicker;
+const {IrcParser} = imports.ircParser;
+const {TabCompletion} = imports.tabCompletion;
 
 const MAX_NICK_UPDATE_TIME = 5; /* s */
 const MAX_LINES = 5;
 
-let _checker = null;
-let _emojiPicker = null;
 
-var ChatEntry = new Lang.Class({
-    Name: 'ChatEntry',
-    Extends: Gtk.Entry,
-    Implements: [PasteManager.DropTargetIface],
+var ChatEntry = GObject.registerClass({
+    Implements: [DropTargetIface],
     Properties: {
-        'can-drop': GObject.ParamSpec.override('can-drop', PasteManager.DropTargetIface),
+        'can-drop': GObject.ParamSpec.override('can-drop', DropTargetIface),
     },
     Signals: { 'text-pasted': { param_types: [GObject.TYPE_STRING,
                                               GObject.TYPE_INT] },
                'image-pasted': { param_types: [GdkPixbuf.Pixbuf.$gtype] },
                'file-pasted': { param_types: [Gio.File.$gtype] } },
+}, class ChatEntry extends Gtk.Entry {
+    static get _checker() {
+        if (!this.__checker)
+            this.__checker = new Gspell.Checker();
+        return this.__checker;
+    }
 
-    _init: function(params) {
-        this.parent(params);
+    static get _emojiPicker() {
+        if (!this.__emojiPicker)
+            this.__emojiPicker = new EmojiPicker();
+        return this.__emojiPicker;
+    }
 
-        PasteManager.DropTargetIface.addTargets(this, this);
+    _init(params) {
+        super._init(params);
+
+        DropTargetIface.addTargets(this, this);
 
         this._emojiPickedId = 0;
 
-        this.connect('icon-press', Lang.bind(this, this._showEmojiPicker));
+        this.connect('icon-press', this._showEmojiPicker.bind(this));
         this.connect('unmap', () => {
             if (this._emojiPickedId)
-                _emojiPicker.disconnect(this._emojiPickedId);
+                ChatEntry._emojiPicker.disconnect(this._emojiPickedId);
             this._emojiPickedId = 0;
         });
 
         let app = Gio.Application.get_default();
         let action = app.lookup_action('show-emoji-picker');
-        action.connect('activate', Lang.bind(this, this._showEmojiPicker));
-
-        if (!_checker)
-            _checker = new Gspell.Checker();
+        action.connect('activate', this._showEmojiPicker.bind(this));
 
         let buffer = Gspell.EntryBuffer.get_from_gtk_entry_buffer(this.buffer);
-        buffer.set_spell_checker (_checker);
+        buffer.set_spell_checker (ChatEntry._checker);
 
         let spellEntry = Gspell.Entry.get_from_gtk_entry(this);
         spellEntry.set_inline_spell_checking(true);
 
         this._useDefaultHandler = false;
-    },
+    }
 
-    _showEmojiPicker: function() {
+    _showEmojiPicker() {
         if (!this.is_sensitive() || !this.get_mapped())
             return;
 
-        if (!_emojiPicker)
-            _emojiPicker = new EmojiPicker.EmojiPicker();
-
         if (!this._emojiPickedId)
-            this._emojiPickedId = _emojiPicker.connect('emoji-picked',
+            this._emojiPickedId = ChatEntry._emojiPicker.connect('emoji-picked',
                 (w, emoji) => {
                     this.delete_selection();
                     let pos = this.insert_text(emoji, -1, this.get_position());
@@ -80,28 +81,28 @@ var ChatEntry = new Lang.Class({
                 });
 
         let rect = this.get_icon_area(Gtk.EntryIconPosition.SECONDARY);
-        _emojiPicker.set_relative_to(this);
-        _emojiPicker.set_pointing_to(rect);
-        _emojiPicker.popup();
-    },
+        ChatEntry._emojiPicker.set_relative_to(this);
+        ChatEntry._emojiPicker.set_pointing_to(rect);
+        ChatEntry._emojiPicker.popup();
+    }
 
     get can_drop() {
         return true;
-    },
+    }
 
-    vfunc_drag_data_received: function(context, x, y, data, info, time) {
+    vfunc_drag_data_received(context, x, y, data, info, time) {
         let str = data.get_text();
         if (!str || str.split('\n').length >= MAX_LINES)
             // Disable GtkEntry's built-in drop target support
             return;
 
          GObject.signal_stop_emission_by_name(this, 'drag-data-received');
-        this.parent(context, x, y, data, info, time);
-    },
+        super.vfunc_drag_data_received(context, x, y, data, info, time);
+    }
 
-    vfunc_paste_clipboard: function(entry) {
+    vfunc_paste_clipboard(entry) {
         if (!this.editable || this._useDefaultHandler) {
-            this.parent();
+            super.vfunc_paste_clipboard();
             return;
         }
 
@@ -110,7 +111,7 @@ var ChatEntry = new Lang.Class({
             if (uris && uris.length)
                 this.emit('file-pasted', Gio.File.new_for_uri(uris[0]));
             else
-                clipboard.request_text(Lang.bind(this, this._onTextReceived));
+                clipboard.request_text(this._onTextReceived.bind(this));
         });
 
         clipboard.request_image((clipboard, pixbuf) => {
@@ -118,9 +119,9 @@ var ChatEntry = new Lang.Class({
                 return;
             this.emit('image-pasted', pixbuf);
         });
-    },
+    }
 
-    _onTextReceived: function(clipboard, text) {
+    _onTextReceived(clipboard, text) {
         if (text == null)
             return;
         text = text.trim();
@@ -137,16 +138,57 @@ var ChatEntry = new Lang.Class({
     }
 });
 
-var EntryArea = new Lang.Class({
-    Name: 'EntryArea',
-    Extends: Gtk.Stack,
+var NickPopover = GObject.registerClass({
+    Template: 'resource:///org/gnome/Polari/ui/nick-popover.ui',
+    InternalChildren: ['nickEntry',
+                       'changeButton'],
+    Properties: {
+        nick: GObject.ParamSpec.string('nick',
+                                       'nick',
+                                       'nick',
+                                       GObject.ParamFlags.READWRITE |
+                                       GObject.ParamFlags.EXPLICIT_NOTIFY,
+                                       '')
+    },
+    Signals: { 'nick-changed': {} }
+}, class NickPopover extends Gtk.Popover {
+    _init() {
+        this._nick = '';
+
+        super._init();
+
+        this.set_default_widget(this._changeButton);
+
+        this._changeButton.connect('clicked', () => {
+            if (!this._nickEntry.text)
+                return;
+
+            this._nick = this._nickEntry.text;
+            this.emit('nick-changed');
+        });
+    }
+
+    get nick() {
+        return this._nick;
+    }
+
+    set nick(nick) {
+        if (this._nick == nick)
+            return;
+
+        if (!this._nickEntry['is-focus'])
+            this._nickEntry.text = nick;
+        this._nick = nick;
+
+        this.notify('nick');
+    }
+});
+
+var EntryArea = GObject.registerClass({
     Template: 'resource:///org/gnome/Polari/ui/entry-area.ui',
     InternalChildren: ['chatEntry',
                        'nickButton',
                        'nickLabel',
-                       'nickPopover',
-                       'nickEntry',
-                       'changeButton',
                        'pasteBox',
                        'confirmLabel',
                        'uploadLabel',
@@ -159,22 +201,47 @@ var EntryArea = new Lang.Class({
                                                  GObject.ParamFlags.WRITABLE,
                                                  0, GLib.MAXUINT32, 0)
     },
+}, class EntryArea extends Gtk.Stack {
+    static get _nickPopover() {
+        if (!this.__nickPopover)
+            this.__nickPopover = new NickPopover();
+        return this.__nickPopover;
+    }
 
-    _init: function(params) {
+    _init(params) {
         this._room = params.room;
         delete params.room;
 
-        this._ircParser = new IrcParser.IrcParser(this._room);
+        this._ircParser = new IrcParser(this._room);
         this._maxNickChars = ChatView.MAX_NICK_CHARS;
+        this._nickChangedId = 0;
 
-        this.parent(params);
+        super._init(params);
 
-        this.connect('destroy', Lang.bind(this, this._onDestroy));
-        this.connect('notify::sensitive', Lang.bind(this, this._onSensitiveChanged));
+        this.connect('destroy', this._onDestroy.bind(this));
+        this.connect('notify::sensitive', this._onSensitiveChanged.bind(this));
         this.connect('realize', () => {
             this._toplevel = this.get_toplevel();
             this._keyPressId = this._toplevel.connect('key-press-event',
-                                                      Lang.bind(this, this._onKeyPressEvent));
+                                                      this._onKeyPressEvent.bind(this));
+        });
+        this.connect('map', () => {
+            this._nickButton.popover = EntryArea._nickPopover;
+
+            if (this._nickChangedId)
+                return;
+
+            this._nickChangedId = EntryArea._nickPopover.connect('nick-changed',
+                () => {
+                    this._setNick(EntryArea._nickPopover.nick);
+                    this._nickButton.active = false;
+                });
+            this._updateNick();
+        });
+        this.connect('unmap', () => {
+            if (this._nickChangedId)
+                EntryArea._nickPopover.disconnect(this._nickChangedId);
+            this._nickChangedId = 0;
         });
 
         this._nickLabel.set_state_flags(Gtk.StateFlags.LINK, false);
@@ -191,13 +258,6 @@ var EntryArea = new Lang.Class({
             w.set_state_flags (state, true);
         });
 
-        this._changeButton.connect('clicked', () => {
-           if (this._nickEntry.text)
-               this._setNick(this._nickEntry.text);
-           this._nickButton.active = false;
-        });
-        this._nickPopover.set_default_widget(this._changeButton);
-
         this._chatEntry.connect('text-pasted', (entry, text, nLines) => {
             this.pasteText(text, nLines);
         });
@@ -219,7 +279,7 @@ var EntryArea = new Lang.Class({
             this.pasteFile(file);
         });
 
-        this._chatEntry.connect('changed', Lang.bind(this, this._onEntryChanged));
+        this._chatEntry.connect('changed', this._onEntryChanged.bind(this));
 
         this._chatEntry.connect('activate', () => {
             if (this._ircParser.process(this._chatEntry.text)) {
@@ -230,8 +290,8 @@ var EntryArea = new Lang.Class({
             }
         });
 
-        this._cancelButton.connect('clicked', Lang.bind(this, this._onCancelClicked));
-        this._pasteButton.connect('clicked', Lang.bind(this, this._onPasteClicked));
+        this._cancelButton.connect('clicked', this._onCancelClicked.bind(this));
+        this._pasteButton.connect('clicked', this._onPasteClicked.bind(this));
 
         this._pasteBox.connect_after('key-press-event', (w, event) => {
             let [, keyval] = event.get_keyval();
@@ -248,28 +308,30 @@ var EntryArea = new Lang.Class({
         if (!this._room)
             return;
 
-        this._completion = new TabCompletion.TabCompletion(this._chatEntry);
+        this._completion = new TabCompletion(this._chatEntry);
         this._membersChangedId =
             this._room.connect('members-changed',
-                               Lang.bind(this, this._updateCompletions));
+                               this._updateCompletions.bind(this));
         this._nicknameChangedId =
-            this._room.account.connect('notify::nickname',
-                                       Lang.bind(this, this._updateNick));
+            this._room.account.connect('notify::nickname', () => {
+                if (!this._room.channel)
+                    this._updateNick();
+            });
         this._channelChangedId =
             this._room.connect('notify::channel',
-                               Lang.bind(this, this._onChannelChanged));
+                               this._onChannelChanged.bind(this));
         this._onChannelChanged(this._room);
 
-        this._chatEntry.connect('map', Lang.bind(this, this._updateCompletions));
-        this._chatEntry.connect('unmap', Lang.bind(this, this._updateCompletions));
-    },
+        this._chatEntry.connect('map', this._updateCompletions.bind(this));
+        this._chatEntry.connect('unmap', this._updateCompletions.bind(this));
+    }
 
     set max_nick_chars(maxChars) {
         this._maxNickChars = maxChars;
         this._updateNick();
-    },
+    }
 
-    _updateCompletions: function() {
+    _updateCompletions() {
         let nicks = [];
 
         if (this._chatEntry.get_mapped() &&
@@ -280,17 +342,17 @@ var EntryArea = new Lang.Class({
             nicks = members.map(member => member.alias);
         }
         this._completion.setCompletions(nicks);
-    },
+    }
 
-    _canFocusChatEntry: function() {
+    _canFocusChatEntry() {
         let toplevelFocus = this._chatEntry.get_toplevel().get_focus();
         return this.sensitive &&
                this._chatEntry.get_mapped() &&
                !this._chatEntry['has-focus'] &&
                !(toplevelFocus instanceof Gtk.Entry);
-    },
+    }
 
-    _onKeyPressEvent: function(w, event) {
+    _onKeyPressEvent(w, event) {
         if (!this._canFocusChatEntry())
             return Gdk.EVENT_PROPAGATE;
 
@@ -308,19 +370,19 @@ var EntryArea = new Lang.Class({
             Gdk.KEY_ISO_Enter,
             Gdk.KEY_space
         ];
-        if (activationKeys.indexOf(keyval) != -1)
+        if (activationKeys.includes(keyval))
             return Gdk.EVENT_PROPAGATE;
 
         this._chatEntry.grab_focus_without_selecting();
         this._chatEntry.event(event);
         return Gdk.EVENT_STOP;
-    },
+    }
 
-    _onEntryChanged: function() {
+    _onEntryChanged() {
         this._chatEntry.get_style_context().remove_class('error');
-    },
+    }
 
-    _setPasteContent: function(content) {
+    _setPasteContent(content) {
         this._pasteContent = content;
 
         if (content) {
@@ -331,9 +393,9 @@ var EntryArea = new Lang.Class({
             this.visible_child_name = 'default';
             this._chatEntry.grab_focus_without_selecting();
         }
-    },
+    }
 
-    pasteText: function(text, nLines) {
+    pasteText(text, nLines) {
         this._confirmLabel.label =
             ngettext("Paste %s line of text to public paste service?",
                      "Paste %s lines of text to public paste service?",
@@ -343,22 +405,22 @@ var EntryArea = new Lang.Class({
                      "Uploading %s lines of text to public paste service…",
                      nLines).format(nLines);
         this._setPasteContent(text);
-    },
+    }
 
-    pasteImage: function(pixbuf) {
+    pasteImage(pixbuf) {
         this._confirmLabel.label = _("Upload image to public paste service?");
         this._uploadLabel.label = _("Uploading image to public paste service…");
         this._setPasteContent(pixbuf);
-    },
+    }
 
-    pasteFile: function(file) {
+    pasteFile(file) {
         file.query_info_async(Gio.FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME,
                               Gio.FileQueryInfoFlags.NONE,
                               GLib.PRIORITY_DEFAULT, null,
-                              Lang.bind(this, this._onFileInfoReady));
-    },
+                              this._onFileInfoReady.bind(this));
+    }
 
-    _onFileInfoReady: function(file, res) {
+    _onFileInfoReady(file, res) {
         let fileInfo = null;
         try {
             fileInfo = file.query_info_finish(res);
@@ -372,9 +434,9 @@ var EntryArea = new Lang.Class({
         /* Translators: %s is a filename */
         this._uploadLabel.label = _("Uploading “%s” to public paste service…").format(name);
         this._setPasteContent(file);
-    },
+    }
 
-    _onPasteClicked: function() {
+    _onPasteClicked() {
         let title;
         let nick = this._room.channel.connection.self_contact.alias;
         if (this._room.type == Tp.HandleType.ROOM)
@@ -397,34 +459,36 @@ var EntryArea = new Lang.Class({
                   (type == 'object' ? this._pasteContent.toString() : type));
         }
         this._confirmLabel.hide();
-    },
+    }
 
-    _onCancelClicked: function() {
+    _onCancelClicked() {
         this._setPasteContent(null);
-    },
+    }
 
-    _onSensitiveChanged: function() {
+    _onSensitiveChanged() {
         if (this._canFocusChatEntry())
             this._chatEntry.grab_focus();
-    },
+    }
 
-    _onChannelChanged: function(room) {
+    _onChannelChanged(room) {
         this._updateCompletions();
 
         if (room.channel)
             this._selfAliasChangedId =
                 room.channel.connection.connect('notify::self-contact',
-                                                Lang.bind(this, this._updateNick));
+                                                this._updateNick.bind(this));
         else
             this._selfAliasChangedId = 0;
         this._updateNick();
-    },
+    }
 
-
-    _setNick: function(nick) {
+    _setNick(nick) {
         this._nickLabel.width_chars = Math.max(nick.length, this._maxNickChars);
         this._nickLabel.label = nick;
 
+        if (!this.get_mapped())
+            return;
+
         let app = Gio.Application.get_default();
         app.setAccountNick(this._room.account, nick);
 
@@ -440,9 +504,9 @@ var EntryArea = new Lang.Class({
             this._updateNick();
             return GLib.SOURCE_REMOVE;
         });
-    },
+    }
 
-    _updateNick: function() {
+    _updateNick() {
         let channel = this._room ? this._room.channel : null;
         let nick = channel ? channel.connection.self_contact.alias
                            : this._room ? this._room.account.nickname : '';
@@ -450,11 +514,11 @@ var EntryArea = new Lang.Class({
         this._nickLabel.width_chars = Math.max(nick.length, this._maxNickChars);
         this._nickLabel.label = nick;
 
-        if (!this._nickEntry['is-focus'])
-            this._nickEntry.text = nick;
-    },
+        if (this.get_mapped())
+            EntryArea._nickPopover.nick = nick;
+    }
 
-    _onDestroy: function() {
+    _onDestroy() {
         if (this._membersChangedId)
             this._room.disconnect(this._membersChangedId);
         this._membersChangedId = 0;
@@ -470,5 +534,8 @@ var EntryArea = new Lang.Class({
         if (this._keyPressId)
             this._toplevel.disconnect(this._keyPressId);
         this._keyPressId = 0;
+        if (this._nickChangedId)
+            EntryArea._nickPopover.disconnect(this._nickChangedId);
+        this._nickChangedId = 0;
     }
 });
