@@ -1,8 +1,8 @@
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
+const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
 
-const Lang = imports.lang;
 const Utils = imports.utils;
 
 const SetupPage = {
@@ -11,19 +11,17 @@ const SetupPage = {
     OFFLINE: 2
 };
 
-var InitialSetupWindow = new Lang.Class({
-    Name: 'InitialSetupWindow',
-    Extends: Gtk.Window,
+var InitialSetupWindow = GObject.registerClass({
     Template: 'resource:///org/gnome/Polari/ui/initial-setup-window.ui',
     InternalChildren: ['contentStack',
                        'connectionsList',
                        'nextButton',
                        'prevButton',
                        'serverRoomList'],
+}, class InitialSetupWindow extends Gtk.Window {
+    _init(params) {
 
-    _init: function(params) {
-
-        this.parent(params);
+        super._init(params);
 
         this._currentAccount = null;
 
@@ -36,7 +34,7 @@ var InitialSetupWindow = new Lang.Class({
         this.connect('destroy', () => { this._unsetAccount(); });
 
         this._serverRoomList.connect('notify::can-join',
-                                     Lang.bind(this, this._updateNextSensitivity));
+                                     this._updateNextSensitivity.bind(this));
 
         this._nextButton.connect('clicked', () => {
             if (this._page == SetupPage.CONNECTION) {
@@ -59,19 +57,19 @@ var InitialSetupWindow = new Lang.Class({
 
         this._networkMonitor = Gio.NetworkMonitor.get_default();
         this._networkMonitor.connect('notify::network-available',
-                                     Lang.bind(this, this._onNetworkAvailableChanged));
+                                     this._onNetworkAvailableChanged.bind(this));
         this._onNetworkAvailableChanged();
-    },
+    }
 
-    _onNetworkAvailableChanged: function() {
+    _onNetworkAvailableChanged() {
         if (this._networkMonitor.network_available)
             this._setPage(this._currentAccount ? SetupPage.ROOM
                                                : SetupPage.CONNECTION);
         else
             this._setPage(SetupPage.OFFLINE);
-    },
+    }
 
-    _setPage: function(page) {
+    _setPage(page) {
         if (page == SetupPage.CONNECTION)
             this._contentStack.visible_child_name = 'connections';
         else if (page == SetupPage.ROOM)
@@ -92,9 +90,9 @@ var InitialSetupWindow = new Lang.Class({
 
         this._nextButton.grab_default();
         this._updateNextSensitivity();
-    },
+    }
 
-    _unsetAccount: function() {
+    _unsetAccount() {
         if (!this._currentAccount)
             return;
 
@@ -102,7 +100,7 @@ var InitialSetupWindow = new Lang.Class({
             a.remove_finish(res);
         });
         this._currentAccount = null;
-    },
+    }
 
     get _page() {
         if (this._contentStack.visible_child_name == 'rooms')
@@ -111,18 +109,18 @@ var InitialSetupWindow = new Lang.Class({
             return SetupPage.CONNECTION;
         else
             return SetupPage.OFFLINE;
-    },
+    }
 
-    _updateNextSensitivity: function() {
+    _updateNextSensitivity() {
         let sensitive = this._page != SetupPage.OFFLINE;
 
         if (this._page == SetupPage.ROOM)
             sensitive = this._serverRoomList.can_join;
 
         this._nextButton.sensitive = sensitive;
-    },
+    }
 
-    _joinRooms: function() {
+    _joinRooms() {
         this.hide();
 
         let toJoinRooms = this._serverRoomList.selectedRooms;
