@@ -1,11 +1,10 @@
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
+const Signals = imports.signals;
 const Tp = imports.gi.TelepathyGLib;
 
 const AppNotifications = imports.appNotifications;
-const Lang = imports.lang;
-const RoomManager = imports.roomManager;
-const Signals = imports.signals;
+const {RoomManager} = imports.roomManager;
 const Utils = imports.utils;
 
 const N_ = s => s;
@@ -39,28 +38,26 @@ var knownCommands = {
 const UNKNOWN_COMMAND_MESSAGE =
     N_("Unknown command — try /HELP for a list of available commands");
 
-var IrcParser = new Lang.Class({
-    Name: 'IrcParser',
-
-    _init: function(room) {
+var IrcParser = class {
+    constructor(room) {
         this._app = Gio.Application.get_default();
         this._roomManager = RoomManager.getDefault();
         this._room = room;
-    },
+    }
 
-    _createFeedbackLabel: function(text) {
+    _createFeedbackLabel(text) {
         return new AppNotifications.SimpleOutput(text);
-    },
+    }
 
-    _createFeedbackUsage: function(cmd) {
+    _createFeedbackUsage(cmd) {
         return this._createFeedbackLabel(_("Usage: %s").format(_(knownCommands[cmd])));
-    },
+    }
 
-    _createFeedbackGrid: function(header, items) {
+    _createFeedbackGrid(header, items) {
         return new AppNotifications.GridOutput(header, items);
-    },
+    }
 
-    process: function(text) {
+    process(text) {
         if (!this._room || !this._room.channel || !text.length)
             return true;
 
@@ -279,15 +276,15 @@ var IrcParser = new Lang.Class({
         if (output)
             this._app.commandOutputQueue.addNotification(output);
         return retval;
-    },
+    }
 
-    _sendText: function(text) {
+    _sendText(text) {
         let type = Tp.ChannelTextMessageType.NORMAL;
         let message = Tp.ClientMessage.new_text(type, text);
         this._sendMessage(message);
-    },
+    }
 
-    _sendMessage: function(message) {
+    _sendMessage(message) {
         this._room.channel.send_message_async(message, 0, (c, res) => {
             try {
                  c.send_message_finish(res);
@@ -297,5 +294,5 @@ var IrcParser = new Lang.Class({
             }
         });
     }
-});
+};
 Signals.addSignalMethods(IrcParser.prototype);
