@@ -15,9 +15,10 @@ pkg.requireSymbol('GLib', '2.0', 'log_variant');
 pkg.requireSymbol('Gspell', '1', 'Entry');
 pkg.requireSymbol('Gtk', '3.0', 'ScrolledWindow.propagate_natural_width');
 
-const Application = imports.application;
 const GLib = imports.gi.GLib;
 
+const {Application} = imports.application;
+
 var LOG_DOMAIN = 'Polari';
 
 function _makeLogFunction(level) {
@@ -48,7 +49,7 @@ function main(args) {
     if (GLib.log_writer_is_journald(2))
         GLib.setenv('G_MESSAGES_DEBUG', LOG_DOMAIN, false);
 
-    let application = new Application.Application();
+    let application = new Application();
     if (GLib.getenv('POLARI_PERSIST'))
         application.hold();
     return application.run(args);
