@@ -1,21 +1,15 @@
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
-
-const Lang = imports.lang;
 const Signals = imports.signals;
 
-let _singleton = null;
+var NetworksManager = class {
+    static getDefault() {
+        if (!this._singleton)
+            this._singleton = new NetworksManager();
+        return this._singleton;
+    }
 
-function getDefault() {
-    if (_singleton == null)
-        _singleton = new NetworksManager();
-    return _singleton;
-}
-
-var NetworksManager = new Lang.Class({
-    Name: 'NetworksManager',
-
-    _init: function() {
+    constructor() {
         this._networks = [];
         this._networksById = new Map();
 
@@ -28,9 +22,9 @@ var NetworksManager = new Lang.Class({
         } catch(e) {
             log('Failed to load network list: ' + e.message);
         }
-    },
+    }
 
-    _onContentsReady: function(f, res) {
+    _onContentsReady(f, res) {
         let success, data;
         try {
             [success, data, ] = f.load_contents_finish(res);
@@ -40,9 +34,9 @@ var NetworksManager = new Lang.Class({
         }
         if (this._parseNetworks(data))
             this.emit('changed');
-    },
+    }
 
-    _parseNetworks: function(data) {
+    _parseNetworks(data) {
         let networks;
         try {
             networks = JSON.parse(data);
@@ -57,37 +51,37 @@ var NetworksManager = new Lang.Class({
             this._networksById.set(network.id, network);
         });
         return true;
-    },
+    }
 
-    _lookupNetwork: function(id) {
+    _lookupNetwork(id) {
         let network = this._networksById.get(id);
         if (!network)
             throw new Error('Invalid network ID');
         return network;
-    },
+    }
 
     get networks() {
         return this._networks;
-    },
+    }
 
-    getAccountIsPredefined: function(account) {
+    getAccountIsPredefined(account) {
         return account && this._networksById.get(account.service) != null;
-    },
+    }
 
-    getNetworkName: function(id) {
+    getNetworkName(id) {
         return this._lookupNetwork(id).name;
-    },
+    }
 
-    getNetworkIsFavorite: function(id) {
+    getNetworkIsFavorite(id) {
         let network = this._lookupNetwork(id);
 
         if (network.hasOwnProperty('favorite'))
             return network['favorite'];
 
         return false;
-    },
+    }
 
-    getNetworkDetails: function(id) {
+    getNetworkDetails(id) {
         let network = this._lookupNetwork(id);
         if (!network.servers || !network.servers.length)
             throw new Error('No servers for network ' + id);
@@ -99,27 +93,27 @@ var NetworksManager = new Lang.Class({
             'port': new GLib.Variant('u', server.port),
             'use-ssl': new GLib.Variant('b', server.ssl)
         };
-    },
+    }
 
-    getNetworkServers: function(id) {
+    getNetworkServers(id) {
         let network = this._lookupNetwork(id);
         let sslServers = network.servers.filter(s => s.ssl);
         return sslServers.length > 0 ? sslServers
                                      : network.servers.slice();
-    },
+    }
 
-    getNetworkMatchTerms: function(id) {
+    getNetworkMatchTerms(id) {
         let network = this._lookupNetwork(id);
         let servers = network.servers.map(s => s.address.toLowerCase());
         return [network.name.toLowerCase(),
                 network.id.toLowerCase()].concat(servers);
-    },
+    }
 
-    findByServer: function(server) {
+    findByServer(server) {
         for (let n of this._networks)
            if (n.servers.some(s => s.address == server))
                return n.id;
         return null;
     }
-});
+};
 Signals.addSignalMethods(NetworksManager.prototype);
