@@ -1,14 +1,11 @@
 const Gdk = imports.gi.Gdk;
 const GdkPixbuf = imports.gi.GdkPixbuf;
-const GLib = imports.gi.GLib;
 const Gio = imports.gi.Gio;
+const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
 const Polari = imports.gi.Polari;
-const Tp = imports.gi.TelepathyGLib;
 
-const AppNotifications = imports.appNotifications;
-const Lang = imports.lang;
 const Utils = imports.utils;
 
 const DndTargetType = {
@@ -28,13 +25,8 @@ function _getTargetForContentType(contentType) {
 }
 
 
-var PasteManager = new Lang.Class({
-    Name: 'PasteManager',
-
-    _init: function() {
-    },
-
-    pasteContent: function(content, title, callback) {
+var PasteManager = class {
+    pasteContent(content, title, callback) {
         if (typeof content == 'string') {
             Utils.gpaste(content, title, callback);
         } else if (content instanceof GdkPixbuf.Pixbuf) {
@@ -44,16 +36,17 @@ var PasteManager = new Lang.Class({
         } else {
             throw new Error('Unhandled content type');
         }
-    },
+    }
 
-    _pasteFile: function(file, title, callback) {
+    _pasteFile(file, title, callback) {
         file.query_info_async(Gio.FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
                               Gio.FileQueryInfoFlags.NONE,
-                              GLib.PRIORITY_DEFAULT, null,
-                              Lang.bind(this, this._onFileQueryFinish, title, callback));
-    },
+                              GLib.PRIORITY_DEFAULT, null, (file, res) => {
+                                  this._onFileQueryFinish(file, res, title, callback);
+                              });
+    }
 
-    _onFileQueryFinish: function(file, res, title, callback) {
+    _onFileQueryFinish(file, res, title, callback) {
         let fileInfo = null;
         try {
             fileInfo = file.query_info_finish(res);
@@ -80,10 +73,9 @@ var PasteManager = new Lang.Class({
         else
             callback(null);
     }
-});
+};
 
-var DropTargetIface = new Lang.Interface({
-    Name: 'DropTargetIface',
+var DropTargetIface = GObject.registerClass({
     Requires: [GObject.Object],
     Properties: {
         'can-drop': GObject.ParamSpec.boolean('can-drop', '', '',
@@ -95,8 +87,8 @@ var DropTargetIface = new Lang.Interface({
         'image-dropped': { param_types: [GdkPixbuf.Pixbuf.$gtype] },
         'file-dropped': { param_types: [Gio.File.$gtype] }
     },
-
-    addTargets: function(widget) {
+}, class DropTargetIface extends GObject.Interface {
+    addTargets(widget) {
         this._dragHighlight = false;
 
         widget.drag_dest_set(0, [], Gdk.DragAction.COPY);
@@ -111,34 +103,34 @@ var DropTargetIface = new Lang.Interface({
 
         widget.drag_dest_set_target_list(targetList);
 
-        widget.connect('drag-drop', Lang.bind(this, this._onDragDrop));
-        widget.connect('drag-leave', Lang.bind(this, this._onDragLeave));
-        widget.connect('drag-motion', Lang.bind(this, this._onDragMotion));
+        widget.connect('drag-drop', this._onDragDrop.bind(this));
+        widget.connect('drag-leave', this._onDragLeave.bind(this));
+        widget.connect('drag-motion', this._onDragMotion.bind(this));
         widget.connect_after('drag-data-received',
-                             Lang.bind(this, this._onDragDataReceived));
-    },
+                             this._onDragDataReceived.bind(this));
+    }
 
-    _onDragDrop: function(widget, context, x, y, time) {
+    _onDragDrop(widget, context, x, y, time) {
         if (!this.can_drop)
             return Gdk.EVENT_PROPAGATE;
 
-        if (!Polari.drag_dest_supports_target(widget, context, null))
+        if (!Polari.drag_dest_supports_target(widget, context))
             return Gdk.EVENT_PROPAGATE;
 
         Polari.drag_dest_request_data(widget, context, time);
         return Gdk.EVENT_STOP;
-    },
+    }
 
-    _onDragLeave: function(widget, context, time) {
+    _onDragLeave(widget, context, time) {
         widget.drag_unhighlight();
         this._dragHighlight = false;
-    },
+    }
 
-    _onDragMotion: function(widget, context, x, y, time) {
+    _onDragMotion(widget, context, x, y, time) {
         if (!this.can_drop)
             return Gdk.EVENT_PROPAGATE;
 
-        if (!Polari.drag_dest_supports_target(widget, context, null))
+        if (!Polari.drag_dest_supports_target(widget, context))
             return Gdk.EVENT_PROPAGATE;
 
         let info = Polari.drag_dest_find_target(widget, context);
@@ -158,10 +150,10 @@ var DropTargetIface = new Lang.Interface({
         }
 
         return Gdk.EVENT_STOP;
-    },
+    }
 
 
-    _onDragDataReceived: function(widget, context, x, y, data, info, time) {
+    _onDragDataReceived(widget, context, x, y, data, info, time) {
         if (info == DndTargetType.URI_LIST) {
             let uris = data.get_uris();
             if (!uris) {
@@ -195,9 +187,9 @@ var DropTargetIface = new Lang.Interface({
             }
             Gtk.drag_finish(context, success, false, time);
         }
-    },
+    }
 
-    _lookupFileInfo: function(file, callback) {
+    _lookupFileInfo(file, callback) {
         let attr = Gio.FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE;
         let flags = Gio.FileQueryInfoFlags.NONE;
         let priority = GLib.PRIORITY_DEFAULT;
