@@ -9,6 +9,8 @@ const Tp = imports.gi.TelepathyGLib;
 const {AccountsMonitor} = imports.accountsMonitor;
 const {RoomManager} = imports.roomManager;
 
+const MIN_SPINNER_TIME = 1000000;   //in microsecond
+
 function _onPopoverVisibleChanged(popover) {
     let context = popover.relative_to.get_style_context();
     if (popover.visible)
@@ -27,6 +29,9 @@ var RoomRow = GObject.registerClass({
         this._room = room;
         this._popover = null;
 
+        this._popoverVisibleChangedId = 0;
+        this._connectingTimeoutId = 0;
+
         this._icon.gicon = room.icon;
         this._icon.visible = room.icon != null;
 
@@ -34,21 +39,37 @@ var RoomRow = GObject.registerClass({
                                this._onButtonRelease.bind(this));
         this.connect('key-press-event', this._onKeyPress.bind(this));
 
-        room.connect('notify::channel', this._onChannelChanged.bind(this));
         room.bind_property('display-name', this._roomLabel, 'label',
                            GObject.BindingFlags.SYNC_CREATE);
 
+        let channelChangedId =
+            room.connect('notify::channel', this._onChannelChanged.bind(this));
+
+        let connectionStatusChangedId = 0;
+
         if (this._room.type == Tp.HandleType.ROOM) {
-            let connectionStatusChangedId =
+            connectionStatusChangedId =
                 this.account.connect('notify::connection-status',
                                      this._onConnectionStatusChanged.bind(this));
             this._onConnectionStatusChanged();
-
-            this.connect('destroy', () => {
-                this.account.disconnect(connectionStatusChangedId);
-            });
         }
 
+        this.connect('destroy', () => {
+            if (this._popoverVisibleChangedId)
+                this._popover.disconnect(this._popoverVisibleChangedId);
+            this._popoverVisibleChangedId = 0;
+
+            room.disconnect(channelChangedId);
+            this._channelSignals.forEach(id => {
+                room.channel.disconnect(id);
+            });
+            if (connectionStatusChangedId)
+                this.account.disconnect(connectionStatusChangedId);
+            if (this._connectingTimeoutId)
+                Mainloop.source_remove(this._connectingTimeoutId);
+            this._connectingTimeoutId = 0;
+        });
+
         this._updatePending();
         this._onChannelChanged();
         this._eventStack.visible_child_name = 'messages';
@@ -98,7 +119,9 @@ var RoomRow = GObject.registerClass({
         let status = this._getConnectionStatus();
         // Show loading indicator if joining a room takes more than 3 seconds
         if (status == Tp.ConnectionStatus.CONNECTED && !this._room.channel)
-            Mainloop.timeout_add_seconds(3, () => {
+            this._connectingTimeoutId = Mainloop.timeout_add_seconds(3, () => {
+                this._connectingTimeoutId = 0;
+
                 if (this._room.channel)
                     return GLib.SOURCE_REMOVE;
                 this._eventStack.visible_child_name = 'connecting';
@@ -122,13 +145,17 @@ var RoomRow = GObject.registerClass({
     }
 
     _onChannelChanged() {
+        this._channelSignals = [];
+
         if (!this._room.channel)
             return;
         this._eventStack.visible_child_name = 'messages';
-        this._room.channel.connect('message-received',
-                                   this._updatePending.bind(this));
-        this._room.channel.connect('pending-message-removed',
-                                   this._updatePending.bind(this));
+
+        for (let signal of ['message-received', 'pending-message-removed'])
+            this._channelSignals.push(
+                this._room.channel.connect(signal,
+                                           this._updatePending.bind(this))
+            );
         this._updatePending();
     }
 
@@ -163,7 +190,9 @@ var RoomRow = GObject.registerClass({
                         'app.leave-room(("%s", ""))'.format(this._room.id));
 
             this._popover = Gtk.Popover.new_from_model(this, menu);
-            this._popover.connect('notify::visible', _onPopoverVisibleChanged);
+            this._popoverVisibleChangedId =
+                this._popover.connect('notify::visible',
+                                      _onPopoverVisibleChanged);
             this._popover.position = Gtk.PositionType.BOTTOM;
         }
         this._popover.show();
@@ -179,6 +208,7 @@ var RoomListHeader = GObject.registerClass({
                        'popoverTitle',
                        'popoverPassword',
                        'popoverConnect',
+                       'popoverDisconnect',
                        'popoverReconnect',
                        'popoverRemove',
                        'popoverProperties',
@@ -204,6 +234,7 @@ var RoomListHeader = GObject.registerClass({
 
         let target = new GLib.Variant('o', this._account.get_object_path());
         this._popoverConnect.action_target = target;
+        this._popoverDisconnect.action_target = target;
         this._popoverReconnect.action_target = target;
         this._popoverRemove.action_target = target;
         this._popoverProperties.action_target = target;
@@ -217,6 +248,8 @@ var RoomListHeader = GObject.registerClass({
             this.popover.hide();
         });
 
+        this._spinnerActivationTime = 0;
+
         let displayNameChangedId =
             this._account.connect('notify::display-name',
                                   this._onDisplayNameChanged.bind(this));
@@ -292,10 +325,24 @@ var RoomListHeader = GObject.registerClass({
         else if (status == Tp.ConnectionStatus.DISCONNECTED)
             child = 'disconnected';
 
+        if (isError && this._spinner.active) {
+            let spinnerTime = GLib.get_monotonic_time() - this._spinnerActivationTime;
+            if (spinnerTime < MIN_SPINNER_TIME) {
+                Mainloop.timeout_add((MIN_SPINNER_TIME - spinnerTime) / 1000, () => {
+                    this._onConnectionStatusChanged();
+                    return GLib.SOURCE_REMOVE;
+                });
+                return;
+            }
+        }
+
         this._iconStack.visible_child_name = child;
         this._spinner.active = (child == 'connecting');
         this._popoverTitle.visible = !isAuth;
 
+        if(this._spinner.active)
+            this._spinnerActivationTime = GLib.get_monotonic_time();
+
         this._popoverTitle.use_markup = isError;
         this._popoverStatus.use_markup = !isError;
 
@@ -325,6 +372,7 @@ var RoomListHeader = GObject.registerClass({
         let presence = this._account.requested_presence_type;
         let offline = presence == Tp.ConnectionPresenceType.OFFLINE;
         this._popoverConnect.visible = offline;
+        this._popoverDisconnect.visible = !offline;
         this._popoverReconnect.visible = !offline;
         this._onConnectionStatusChanged();
     }
@@ -391,10 +439,10 @@ class RoomList extends Gtk.ListBox {
                                       this._accountAdded.bind(this));
         this._accountsMonitor.connect('account-removed',
                                       this._accountRemoved.bind(this));
-        this._accountsMonitor.connect('account-enabled', (mon, account) => {
+        this._accountsMonitor.connect('account-shown', (mon, account) => {
             this._updatePlaceholderVisibility(account);
         });
-        this._accountsMonitor.connect('account-disabled', (mon, account) => {
+        this._accountsMonitor.connect('account-hidden', (mon, account) => {
             this._updatePlaceholderVisibility(account);
         });
         this._accountsMonitor.prepare(() => {
@@ -575,7 +623,7 @@ class RoomList extends Gtk.ListBox {
     }
 
     _updatePlaceholderVisibility(account) {
-        if (!account.enabled) {
+        if (!account.visible) {
             this._placeholders.get(account).hide();
             return;
         }
