@@ -5,15 +5,12 @@ const Gtk = imports.gi.Gtk;
 const Pango = imports.gi.Pango;
 const Tp = imports.gi.TelepathyGLib;
 
-const AccountsMonitor = imports.accountsMonitor;
-const ChatView = imports.chatView;
-const EntryArea = imports.entryArea;
-const Lang = imports.lang;
-const RoomManager = imports.roomManager;
+const {AccountsMonitor} = imports.accountsMonitor;
+const {ChatView} = imports.chatView;
+const {EntryArea} = imports.entryArea;
+const {RoomManager} = imports.roomManager;
 
-var RoomStack = new Lang.Class({
-    Name: 'RoomStack',
-    Extends: Gtk.Stack,
+var RoomStack = GObject.registerClass({
     Properties: {
         'entry-area-height': GObject.ParamSpec.uint('entry-area-height',
                                                     'entry-area-height',
@@ -21,19 +18,17 @@ var RoomStack = new Lang.Class({
                                                     GObject.ParamFlags.READABLE,
                                                     0, GLib.MAXUINT32, 0)
     },
-
-    _init: function(params) {
-        this.parent(params);
+}, class RoomStack extends Gtk.Stack {
+    _init(params) {
+        super._init(params);
 
         this._sizeGroup = new Gtk.SizeGroup({ mode: Gtk.SizeGroupMode.VERTICAL });
         this._rooms = new Map();
 
         this._roomManager = RoomManager.getDefault();
 
-        this._roomManager.connect('room-added',
-                                  Lang.bind(this, this._roomAdded));
-        this._roomManager.connect('room-removed',
-                                  Lang.bind(this, this._roomRemoved));
+        this._roomManager.connect('room-added', this._roomAdded.bind(this));
+        this._roomManager.connect('room-removed', this._roomRemoved.bind(this));
         this._roomManager.rooms.forEach(r => { this._roomAdded(this._roomManager, r); });
 
         this.add_named(new ChatPlaceholder(this._sizeGroup), 'placeholder');
@@ -43,45 +38,45 @@ var RoomStack = new Lang.Class({
             this._entryAreaHeight = rect.height - 1;
             this.notify('entry-area-height');
         });
-    },
+    }
 
-    vfunc_realize: function() {
-        this.parent();
+    vfunc_realize() {
+        super.vfunc_realize();
 
         let toplevel = this.get_toplevel();
 
         toplevel.connect('notify::active-room',
-                         Lang.bind(this, this._activeRoomChanged));
+                         this._activeRoomChanged.bind(this));
         toplevel.connect('active-room-state-changed',
-                         Lang.bind(this, this._updateSensitivity));
+                         this._updateSensitivity.bind(this));
         this._activeRoomChanged();
         this._updateSensitivity();
-    },
+    }
 
     get entry_area_height() {
         return this._entryAreaHeight;
-    },
+    }
 
-    _addView: function(id, view) {
+    _addView(id, view) {
         this._rooms.set(id, view);
         this.add_named(view, id);
-    },
+    }
 
-    _roomAdded: function(roomManager, room) {
+    _roomAdded(roomManager, room) {
         this._addView(room.id, new RoomView(room, this._sizeGroup));
-    },
+    }
 
-    _roomRemoved: function(roomManager, room) {
+    _roomRemoved(roomManager, room) {
         this._rooms.get(room.id).destroy();
         this._rooms.delete(room.id);
-    },
+    }
 
-    _activeRoomChanged: function() {
+    _activeRoomChanged() {
         let room = this.get_toplevel().active_room;
         this.set_visible_child_name(room ? room.id : 'placeholder');
-    },
+    }
 
-    _updateSensitivity: function() {
+    _updateSensitivity() {
         let room = this.get_toplevel().active_room;
         if (!room)
             return;
@@ -90,16 +85,14 @@ var RoomStack = new Lang.Class({
     }
 });
 
-var SavePasswordConfirmationBar = new Lang.Class({
-    Name: 'SavePasswordConfirmationBar',
-    Extends: Gtk.Revealer,
-
-    _init: function(room) {
+var SavePasswordConfirmationBar = GObject.registerClass(
+class SavePasswordConfirmationBar extends Gtk.Revealer {
+    _init(room) {
         this._room = room;
 
-        this.parent({ valign: Gtk.Align.START });
+        super._init({ valign: Gtk.Align.START });
 
-        this.connect('destroy', Lang.bind(this, this._onDestroy));
+        this.connect('destroy', this._onDestroy.bind(this));
 
         this._createWidget();
 
@@ -114,9 +107,9 @@ var SavePasswordConfirmationBar = new Lang.Class({
             }
             this.reveal_child = false;
         });
-    },
+    }
 
-    _createWidget: function() {
+    _createWidget() {
         this._infoBar = new Gtk.InfoBar({ show_close_button: true })
         this.add(this._infoBar);
 
@@ -144,20 +137,18 @@ var SavePasswordConfirmationBar = new Lang.Class({
         box.add(this._subtitleLabel);
 
         this._infoBar.show_all();
-    },
+    }
 
-    _onDestroy: function() {
+    _onDestroy() {
         if (this._identifySentId)
             this._room.disconnect(this._identifySentId);
         this._identifySentId = 0;
     }
 });
 
-var ChatPlaceholder = new Lang.Class({
-    Name: 'ChatPlaceholder',
-    Extends: Gtk.Overlay,
-
-    _init: function(sizeGroup) {
+var ChatPlaceholder = GObject.registerClass(
+class ChatPlaceholder extends Gtk.Overlay {
+    _init(sizeGroup) {
         this._accountsMonitor = AccountsMonitor.getDefault();
 
         let image = new Gtk.Image({ icon_name: 'org.gnome.Polari-symbolic',
@@ -177,7 +168,7 @@ var ChatPlaceholder = new Lang.Class({
         let inputPlaceholder = new Gtk.Box({ valign: Gtk.Align.END });
         sizeGroup.add_widget(inputPlaceholder);
 
-        this.parent();
+        super._init();
         let grid = new Gtk.Grid({ column_homogeneous: true, can_focus: false,
                                   column_spacing: 18, hexpand: true, vexpand: true,
                                   valign: Gtk.Align.CENTER });
@@ -191,12 +182,10 @@ var ChatPlaceholder = new Lang.Class({
     }
 });
 
-var RoomView = new Lang.Class({
-    Name: 'RoomView',
-    Extends: Gtk.Overlay,
-
-    _init: function(room, sizeGroup) {
-        this.parent();
+var RoomView = GObject.registerClass(
+class RoomView extends Gtk.Overlay {
+    _init(room, sizeGroup) {
+        super._init({ name: `RoomView ${room.display_name}` });
 
         let box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL });
         this.add(box);
@@ -204,11 +193,11 @@ var RoomView = new Lang.Class({
         if (room.type == Tp.HandleType.CONTACT)
             this.add_overlay(new SavePasswordConfirmationBar(room));
 
-        this._view = new ChatView.ChatView(room);
+        this._view = new ChatView(room);
         box.add(this._view);
 
-        this._entryArea = new EntryArea.EntryArea({ room: room,
-                                                    sensitive: false });
+        this._entryArea = new EntryArea({ room: room,
+                                          sensitive: false });
         box.add(this._entryArea);
 
         this._view.bind_property('max-nick-chars',
@@ -227,7 +216,7 @@ var RoomView = new Lang.Class({
         });
 
         this.show_all();
-    },
+    }
 
     set inputSensitive(sensitive) {
         this._entryArea.sensitive = sensitive;
