@@ -29,7 +29,7 @@ var ServerRoomManager = class {
         this._accountsMonitor.connect('account-removed',
                                       this._onAccountRemoved.bind(this));
         this._accountsMonitor.prepare(() => {
-            this._accountsMonitor.enabledAccounts.forEach(a => {
+            this._accountsMonitor.visibleAccounts.forEach(a => {
                 this._onAccountStatusChanged(this._accountsMonitor, a);
             });
         });
@@ -130,10 +130,6 @@ var ServerRoomList = GObject.registerClass({
 
         super._init(params);
 
-        this.connect('destroy', () => {
-            this.setAccount(null);
-        });
-
         this._list.model.set_visible_func((model, iter) => {
             let name = model.get_value(iter, RoomListColumn.NAME);
             if (!name)
@@ -186,8 +182,15 @@ var ServerRoomList = GObject.registerClass({
         });
 
         this._manager = ServerRoomManager.getDefault();
-        this._manager.connect('loading-changed',
-                              this._onLoadingChanged.bind(this));
+        let loadingChangedId =
+            this._manager.connect('loading-changed',
+                                  this._onLoadingChanged.bind(this));
+
+        this.connect('destroy', () => {
+            this.setAccount(null);
+
+            this._manager.disconnect(loadingChangedId);
+        });
     }
 
     get can_join() {
