@@ -3,19 +3,16 @@ const Gtk = imports.gi.Gtk;
 const Pango = imports.gi.Pango;
 
 const IrcParser = imports.ircParser;
-const Lang = imports.lang;
 
-var TabCompletion = new Lang.Class({
-    Name: 'TabCompletion',
-
-    _init: function(entry) {
+var TabCompletion = class {
+    constructor(entry) {
         this._entry = entry;
         this._canComplete = false;
         this._key = '';
 
-        this._entry.connect('key-press-event', Lang.bind(this, this._onKeyPress));
-        this._entry.connect('focus-out-event', Lang.bind(this, this._cancel));
-        this._entry.connect('unmap', Lang.bind(this, this._cancel));
+        this._entry.connect('key-press-event', this._onKeyPress.bind(this));
+        this._entry.connect('focus-out-event', this._cancel.bind(this));
+        this._entry.connect('unmap', this._cancel.bind(this));
         this._entry.connect('realize', () => {
             this._popup.set_transient_for(this._entry.get_toplevel());
         });
@@ -30,13 +27,13 @@ var TabCompletion = new Lang.Class({
         this._popup.add(frame);
 
         this._list = new Gtk.ListBox({ selection_mode: Gtk.SelectionMode.SINGLE });
-        this._list.set_filter_func(Lang.bind(this, this._filter));
-        this._list.connect('row-selected', Lang.bind(this, this._onRowSelected));
-        this._list.connect('row-activated', Lang.bind(this, this._stop));
-        this._list.connect('keynav-failed', Lang.bind(this, this._onKeynavFailed));
+        this._list.set_filter_func(this._filter.bind(this));
+        this._list.connect('row-selected', this._onRowSelected.bind(this));
+        this._list.connect('row-activated', this._stop.bind(this));
+        this._list.connect('keynav-failed', this._onKeynavFailed.bind(this));
         frame.add(this._list);
 
-        this._widgetMap = {};
+        this._widgetMap = new Map();
         this._previousWasCommand = false;
 
         let commands = Object.keys(IrcParser.knownCommands);
@@ -50,9 +47,9 @@ var TabCompletion = new Lang.Class({
                                     margin_end: 6 }));
             this._list.add(row);
         }
-    },
+    }
 
-    _showPopup: function() {
+    _showPopup() {
         this._list.show_all();
 
         let [, height] = this._list.get_preferred_height();
@@ -73,9 +70,9 @@ var TabCompletion = new Lang.Class({
         y += allocation.y - height;
         this._popup.move(x, y);
         this._popup.show();
-    },
+    }
 
-    setCompletions: function(completions) {
+    setCompletions(completions) {
         if (this._popup.visible) {
             let id = this._popup.connect('unmap', () => {
                 this._popup.disconnect(id);
@@ -84,14 +81,14 @@ var TabCompletion = new Lang.Class({
             return;
         }
 
-        let widgetMap = {};
+        let widgetMap = new Map();
 
         for (let i = 0; i < completions.length; i++) {
             let nick = completions[i];
-            let row = this._widgetMap[nick];
+            let row = this._widgetMap.get(nick);
 
             if (row) {
-                widgetMap[nick] = row;
+                widgetMap.set(nick, row);
                 this._list.remove(row);
             } else {
                 row = new Gtk.ListBoxRow();
@@ -101,7 +98,7 @@ var TabCompletion = new Lang.Class({
                                         halign: Gtk.Align.START,
                                         margin_start: 6,
                                         margin_end: 6 }));
-                widgetMap[nick] = row;
+                widgetMap.set(nick, row);
             }
         }
 
@@ -114,13 +111,13 @@ var TabCompletion = new Lang.Class({
         });
 
         for (let i = 0; i < completions.length; i++) {
-            let row = this._widgetMap[completions[i]];
+            let row = this._widgetMap.get(completions[i]);
             this._list.add(row);
         }
         this._canComplete = completions.length > 0;
-    },
+    }
 
-    _onKeyPress: function(w, event) {
+    _onKeyPress(w, event) {
         let [, keyval] = event.get_keyval();
 
         if (this._key.length == 0) {
@@ -156,9 +153,9 @@ var TabCompletion = new Lang.Class({
                     keyval == Gdk.KEY_ISO_Enter);
         }
         return Gdk.EVENT_PROPAGATE;
-    },
+    }
 
-    _getRowCompletion: function(row) {
+    _getRowCompletion(row) {
         this._previousWasCommand = this._isCommand;
 
         if (this._isCommand)
@@ -166,35 +163,35 @@ var TabCompletion = new Lang.Class({
         if (this._startPos == 0 || this._isChained)
             return row._text + ': ';
         return row._text;
-    },
+    }
 
-    _onRowSelected: function(w, row) {
+    _onRowSelected(w, row) {
         if (row)
             this._insertCompletion(this._getRowCompletion(row));
-    },
+    }
 
-    _filter: function(row) {
+    _filter(row) {
         if (this._key.length == 0)
             return false;
         return row._casefoldedText.startsWith(this._key);
-    },
+    }
 
-    _insertCompletion: function(completion) {
+    _insertCompletion(completion) {
         let pos = this._entry.get_position();
         this._endPos = this._startPos + completion.length;
         this._entry.delete_text(this._startPos, pos);
         this._entry.insert_text(completion, -1, this._startPos);
         this._entry.set_position(this._endPos);
-    },
+    }
 
-    _setPreviousCompletionChained: function(chained) {
+    _setPreviousCompletionChained(chained) {
         let repl = chained ? ',' : ':';
         let start = this._startPos - 2;
         this._entry.delete_text(start, start + 1);
         this._entry.insert_text(repl, -1, start);
-    },
+    }
 
-    _start: function() {
+    _start() {
         if (!this._canComplete)
             return;
 
@@ -227,9 +224,9 @@ var TabCompletion = new Lang.Class({
             this._list.select_row(visibleRows[0]);
             this._showPopup()
         }
-    },
+    }
 
-    _onKeynavFailed: function(w, dir) {
+    _onKeynavFailed(w, dir) {
         if (this._inHandler)
             return Gdk.EVENT_PROPAGATE;
         let count = dir == Gtk.DirectionType.DOWN ? -1 : 1;
@@ -237,15 +234,15 @@ var TabCompletion = new Lang.Class({
         this._moveSelection(Gtk.MovementStep.BUFFER_ENDS, count);
         this._inHandler = false;
         return Gdk.EVENT_STOP;
-    },
+    }
 
-    _moveSelection: function(movement, count) {
+    _moveSelection(movement, count) {
         this._list.emit('move-cursor', movement, count);
         let row = this._list.get_focus_child();
         this._list.select_row(row);
-    },
+    }
 
-    _stop: function() {
+    _stop() {
         if (this._key.length == 0)
             return;
 
@@ -255,14 +252,14 @@ var TabCompletion = new Lang.Class({
         this._key = '';
 
         this._list.invalidate_filter();
-    },
+    }
 
-    _cancel: function() {
+    _cancel() {
         if (this._key.length == 0)
             return;
         if (this._isChained)
             this._setPreviousCompletionChained(false);
         this._insertCompletion('');
         this._stop();
-    },
-});
+    }
+};
