@@ -143,6 +143,8 @@ class TelepathyClient extends Tp.BaseClient {
               handler: this._onLeaveActivated.bind(this) },
             { name: 'connect-account',
               handler: this._onConnectAccountActivated.bind(this) },
+            { name: 'disconnect-account',
+              handler: this._onDisconnectAccountActivated.bind(this) },
             { name: 'reconnect-account',
               handler: this._onReconnectAccountActivated.bind(this) },
             { name: 'authenticate-account',
@@ -184,10 +186,10 @@ class TelepathyClient extends Tp.BaseClient {
         this._accountsMonitor.connect('account-added', (mon, account) => {
             this._connectAccount(account);
         });
-        this._accountsMonitor.connect('account-enabled', (mon, account) => {
+        this._accountsMonitor.connect('account-shown', (mon, account) => {
             this._connectAccount(account);
         });
-        this._accountsMonitor.enabledAccounts.forEach(a => {
+        this._accountsMonitor.visibleAccounts.forEach(a => {
             this._onAccountStatusChanged(this._accountsMonitor, a);
         });
 
@@ -203,7 +205,7 @@ class TelepathyClient extends Tp.BaseClient {
         debug('Network changed to %s'.format(connected ? 'available'
                                                        : 'unavailable'));
 
-        this._accountsMonitor.enabledAccounts.forEach(a => {
+        this._accountsMonitor.visibleAccounts.forEach(a => {
             this._setAccountPresence(a, presence);
         });
     }
@@ -225,6 +227,9 @@ class TelepathyClient extends Tp.BaseClient {
     }
 
     _setAccountPresence(account, presence) {
+        if (!account.enabled)
+            return;
+
         let statuses = Object.keys(Tp.ConnectionPresenceType).map(s =>
             s.replace(/_/g, '-').toLowerCase()
         );
@@ -337,7 +342,18 @@ class TelepathyClient extends Tp.BaseClient {
     _onConnectAccountActivated(action, parameter) {
         let accountPath = parameter.deep_unpack();
         let account = this._accountsMonitor.lookupAccount(accountPath);
-        this._connectAccount(account);
+        if (account.enabled)
+            this._connectAccount(account);
+        else
+            account.set_enabled_async(true, () => {});
+    }
+
+    _onDisconnectAccountActivated(action, parameter) {
+        let accountPath = parameter.deep_unpack();
+        let account = this._accountsMonitor.lookupAccount(accountPath);
+        account.set_enabled_async(false, () => {
+            this._setAccountPresence(account, Tp.ConnectionPresenceType.OFFLINE);
+        });
     }
 
     _onReconnectAccountActivated(action, parameter) {
