@@ -1,26 +1,21 @@
-const Gdk = imports.gi.Gdk;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
-const Pango = imports.gi.Pango;
-const Tp = imports.gi.TelepathyGLib;
-const Polari = imports.gi.Polari;
-
-const Lang = imports.lang;
 const Mainloop = imports.mainloop;
+const Pango = imports.gi.Pango;
+const Polari = imports.gi.Polari;
+const Tp = imports.gi.TelepathyGLib;
 
 const READWRITE = GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE;
 
 const MAX_USERS_SHOWN = 8;
 const MAX_USERS_WIDTH_CHARS = 17;
 
-var UserListPopover = new Lang.Class({
-    Name: 'UserListPopover',
-    Extends: Gtk.Popover,
-
-    _init: function(params) {
-        this.parent(params);
+var UserListPopover = GObject.registerClass(
+class UserListPopover extends Gtk.Popover {
+    _init(params) {
+        super._init(params);
 
         this._createWidget();
 
@@ -32,17 +27,17 @@ var UserListPopover = new Lang.Class({
         this._revealer.connect('notify::child-revealed', () => {
             this._revealer.transition_duration = 250;
         });
-    },
+    }
 
-    vfunc_realize: function() {
-        this.parent();
+    vfunc_realize() {
+        super.vfunc_realize();
 
         let toplevel = this.get_toplevel();
         toplevel.connect('notify::active-room',
-                         Lang.bind(this, this._activeRoomChanged));
-    },
+                         this._activeRoomChanged.bind(this));
+    }
 
-    _createWidget: function() {
+    _createWidget() {
         this._box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL,
                                   spacing: 6 });
         this.add(this._box);
@@ -54,22 +49,21 @@ var UserListPopover = new Lang.Class({
         this._box.add(this._userListBin);
 
         this._entry = new Gtk.SearchEntry({ primary_icon_name: 'avatar-default-symbolic' });
-        this._entry.connect('search-changed',
-                            Lang.bind(this, this._updateFilter));
+        this._entry.connect('search-changed', this._updateFilter.bind(this));
         this._revealer.add(this._entry);
 
         this._box.show_all();
-    },
+    }
 
-    _activeRoomChanged: function() {
+    _activeRoomChanged() {
         this._entry.text = '';
 
         if (this._userList)
             this._userList.destroy();
         this._userList = null;
-    },
+    }
 
-    _ensureUserList: function() {
+    _ensureUserList() {
         if (this._userList)
             return;
 
@@ -81,29 +75,27 @@ var UserListPopover = new Lang.Class({
         this._userListBin.add(this._userList);
 
         this._userList.vadjustment.connect('changed',
-                                           Lang.bind(this, this._updateEntryVisibility));
+                                           this._updateEntryVisibility.bind(this));
         this._updateEntryVisibility();
-    },
+    }
 
-    _updateEntryVisibility: function() {
+    _updateEntryVisibility() {
         if (!this._userList)
             return;
 
         let reveal = this._entry.text != '' ||
                      this._userList.numRows > MAX_USERS_SHOWN;
         this._revealer.reveal_child = reveal;
-    },
+    }
 
-    _updateFilter: function() {
+    _updateFilter() {
         if (!this._userList)
             return;
         this._userList.setFilter(this._entry.text);
     }
 });
 
-var UserDetails = new Lang.Class({
-    Name: 'UserDetails',
-    Extends: Gtk.Frame,
+var UserDetails = GObject.registerClass({
     Template: 'resource:///org/gnome/Polari/ui/user-details.ui',
     InternalChildren: ['spinnerBox',
                        'spinner',
@@ -122,8 +114,8 @@ var UserDetails = new Lang.Class({
                                                              'notifications-enabled',
                                                              READWRITE,
                                                              false)},
-
-    _init: function(params = {}) {
+}, class UserDetails extends Gtk.Frame {
+    _init(params = {}) {
         let user = params.user;
         delete params.user;
 
@@ -132,21 +124,21 @@ var UserDetails = new Lang.Class({
         this._notificationsEnabled = false;
         this._user = null;
 
-        this.parent(params);
+        super._init(params);
 
         this.user = user;
 
         this._messageButton.connect('clicked',
-                                    Lang.bind(this, this._onMessageButtonClicked));
+                                    this._onMessageButtonClicked.bind(this));
 
         this._updateButtonVisibility();
         this._detailsGrid.hide();
         this._notificationLabel.opacity = this.notifications_enabled ? 1. : 0.;
-    },
+    }
 
     get notifications_enabled() {
         return this._notificationsEnabled;
-    },
+    }
 
     set notifications_enabled(value) {
         if (this._notificationsEnabled == value)
@@ -157,7 +149,7 @@ var UserDetails = new Lang.Class({
         this.notify('notifications-enabled');
 
         this._notificationLabel.opacity = value ? 1. : 0.;
-    },
+    }
 
     set user(user) {
         if (this._user == user)
@@ -170,8 +162,9 @@ var UserDetails = new Lang.Class({
         this._user = user;
 
         if (this._user)
-            this._selfContactChangedId = this._user.connection.connect('notify::self-contact',
-                                                    Lang.bind(this, this._updateButtonVisibility));
+            this._selfContactChangedId =
+                this._user.connection.connect('notify::self-contact',
+                                              this._updateButtonVisibility.bind(this));
 
         if (this.expanded)
             this._expand();
@@ -179,7 +172,7 @@ var UserDetails = new Lang.Class({
         this._updateButtonVisibility();
         this._notificationLabel.visible = this._user == null;
         this._lastLabel.visible = this._user != null;
-    },
+    }
 
     set nickname(nickname) {
         this._nickname = nickname;
@@ -189,11 +182,11 @@ var UserDetails = new Lang.Class({
 
 
         this._updateButtonVisibility();
-    },
+    }
 
     get expanded() {
         return this._expanded;
-    },
+    }
 
     set expanded(v) {
         if (v == this._expanded)
@@ -207,9 +200,9 @@ var UserDetails = new Lang.Class({
             this._unexpand();
 
         this.notify('expanded');
-    },
+    }
 
-    _expand: function() {
+    _expand() {
         this._detailsGrid.visible = this._initialDetailsLoaded;
         this._spinnerBox.visible = !this._initialDetailsLoaded;
         this._spinner.start();
@@ -218,21 +211,21 @@ var UserDetails = new Lang.Class({
 
         if (this._user)
             this._user.request_contact_info_async(this._cancellable,
-                                              Lang.bind(this, this._onContactInfoReady));
+                                                  this._onContactInfoReady.bind(this));
         //TODO: else use this._nickname to query tracker
         else
             this._revealDetails();
-    },
+    }
 
-    _unexpand: function() {
+    _unexpand() {
         this._spinner.stop();
 
         if (this._cancellable)
             this._cancellable.cancel();
         this._cancellable = null;
-    },
+    }
 
-    _formatLast: function(seconds) {
+    _formatLast(seconds) {
         if (seconds < 60)
             return ngettext("%d second ago",
                             "%d seconds ago", seconds).format(seconds);
@@ -260,9 +253,9 @@ var UserDetails = new Lang.Class({
         let months = days / 30;
         return ngettext("%d month ago",
                         "%d months ago", months).format(months);
-    },
+    }
 
-    _onContactInfoReady: function(c, res) {
+    _onContactInfoReady(c, res) {
         this._initialDetailsLoaded = true;
 
         let fn, last;
@@ -287,15 +280,15 @@ var UserDetails = new Lang.Class({
         }
 
         this._revealDetails();
-    },
+    }
 
-    _revealDetails: function() {
+    _revealDetails() {
         this._spinner.stop();
         this._spinnerBox.hide();
         this._detailsGrid.show();
-    },
+    }
 
-    _onMessageButtonClicked: function() {
+    _onMessageButtonClicked() {
         let account = this._user.connection.get_account();
 
         let app = Gio.Application.get_default();
@@ -306,9 +299,9 @@ var UserDetails = new Lang.Class({
                                            this._user.alias,
                                            '',
                                            time ]));
-    },
+    }
 
-    _updateButtonVisibility: function() {
+    _updateButtonVisibility() {
         if (!this._user) {
             this._messageButton.sensitive = false;
 
@@ -325,16 +318,14 @@ var UserDetails = new Lang.Class({
     }
 });
 
-var UserPopover = new Lang.Class({
-    Name: 'UserPopover',
-    Extends: Gtk.Popover,
+var UserPopover = GObject.registerClass({
     Template: 'resource:///org/gnome/Polari/ui/user-popover.ui',
     InternalChildren: ['nickLabel',
                        'statusLabel',
                        'notifyButton',
                        'userDetails'],
-
-    _init: function(params) {
+}, class UserPopover extends Gtk.Popover {
+    _init(params) {
         this._room = params.room;
         delete params.room;
 
@@ -344,7 +335,7 @@ var UserPopover = new Lang.Class({
         this._nickname = null;
         this._basenick = null;
 
-        this.parent(params);
+        super._init(params);
 
         this._nickLabel.set_state_flags(Gtk.StateFlags.LINK, false);
 
@@ -359,7 +350,7 @@ var UserPopover = new Lang.Class({
         });
 
         this.show();
-    },
+    }
 
     set nickname(nickname) {
         if (this._nickname == nickname)
@@ -376,9 +367,9 @@ var UserPopover = new Lang.Class({
         this._notifyButton.action_name = actionName;
 
         this._setBasenick(Polari.util_get_basenick(nickname));
-    },
+    }
 
-    _setBasenick: function(basenick) {
+    _setBasenick(basenick) {
         if (this._basenick == basenick)
             return;
 
@@ -388,11 +379,13 @@ var UserPopover = new Lang.Class({
             this._userTracker.unwatchRoomStatus(this._room, this._roomStatusChangedId);
         this._roomStatusChangedId =
             this._userTracker.watchRoomStatus(this._room, this._basenick,
-                                        Lang.bind(this, this._onNickStatusChanged));
+                                              this._onNickStatusChanged.bind(this));
 
         if (this._globalStatusChangedId > 0)
             this._userTracker.disconnect(this._globalStatusChangedId);
-        this._globalStatusChangedId = this._userTracker.connect("status-changed::" + basenick, Lang.bind(this, this._onStatusChanged));
+        this._globalStatusChangedId =
+            this._userTracker.connect("status-changed::" + basenick,
+                                      this._onStatusChanged.bind(this));
 
         if (this._contactsChangedId > 0)
             this._userTracker.disconnect(this._contactsChangedId);
@@ -402,13 +395,13 @@ var UserPopover = new Lang.Class({
 
         this._onStatusChanged();
         this._updateDetailsContact();
-    },
+    }
 
     get nickname() {
         return this._nickname;
-    },
+    }
 
-    _onStatusChanged: function() {
+    _onStatusChanged() {
         let status = this._userTracker.getNickStatus(this._nickname);
         let roomStatus = this._userTracker.getNickRoomStatus(this._nickname,
                                                              this._room);
@@ -423,25 +416,23 @@ var UserPopover = new Lang.Class({
         this._statusLabel.label = label;
 
         this._nickLabel.sensitive = (status == Tp.ConnectionPresenceType.AVAILABLE);
-    },
+    }
 
-    _updateDetailsContact: function() {
+    _updateDetailsContact() {
         this._userDetails.user = this._userTracker.lookupContact(this._nickname);
-     },
+     }
 
-    _onNickStatusChanged: function(baseNick, status) {
+    _onNickStatusChanged(baseNick, status) {
         this._onStatusChanged();
     }
 });
 
-var UserListRow = new Lang.Class({
-    Name: 'UserListRow',
-    Extends: Gtk.ListBoxRow,
-
-    _init: function(user) {
+const UserListRow = GObject.registerClass(
+class UserListRow extends Gtk.ListBoxRow {
+    _init(user) {
         this._user = user;
 
-        this.parent();
+        super._init({ name: `UserListRow ${user.alias}` });
 
         this._createWidget();
 
@@ -449,31 +440,35 @@ var UserListRow = new Lang.Class({
             this._revealer.reveal_child = false;
         });
         this.connect('state-flags-changed',
-                     Lang.bind(this, this._updateArrowVisibility));
+                     this._updateArrowVisibility.bind(this));
 
         this._revealer.connect('notify::reveal-child',
-                               Lang.bind(this, this._onExpandedChanged));
-    },
+                               this._onExpandedChanged.bind(this));
+    }
 
     get user() {
         return this._user;
-    },
+    }
 
     get expand() {
         return this._revealer.reveal_child;
-    },
+    }
 
     set expand(expand) {
         if (expand)
             this._ensureDetails();
         this._revealer.reveal_child = expand;
-    },
+    }
 
-    _createWidget: function() {
+    _createWidget() {
         let vbox = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL });
         this.add(vbox);
 
-        let hbox = new Gtk.Box({ margin: 4, spacing: 4 });
+        let hbox = new Gtk.Box({ margin_end: 12,
+                                 margin_start: 4,
+                                 margin_top: 4,
+                                 margin_bottom: 4,
+                                 spacing: 4 });
         this._arrow = new Gtk.Arrow({ arrow_type: Gtk.ArrowType.RIGHT,
                                       no_show_all: true });
         this._label = new Gtk.Label({ label: this._user.alias,
@@ -490,9 +485,9 @@ var UserListRow = new Lang.Class({
         vbox.add(this._revealer);
 
         this.show_all();
-    },
+    }
 
-    _ensureDetails: function() {
+    _ensureDetails() {
         if (this._revealer.get_child())
             return;
 
@@ -501,18 +496,18 @@ var UserListRow = new Lang.Class({
         this._revealer.bind_property('reveal-child', details, 'expanded', 0);
 
         this._revealer.add(details);
-    },
+    }
 
-    shouldShow: function() {
-        return this._user.alias.toLowerCase().indexOf(this._filter) != -1;
-    },
+    shouldShow() {
+        return this._user.alias.toLowerCase().includes(this._filter);
+    }
 
-    setFilter: function(filter) {
+    setFilter(filter) {
         this._filter = filter.toLowerCase();
         this._updateLabel();
-    },
+    }
 
-    _updateLabel: function() {
+    _updateLabel() {
         let filterIndex = -1;
         if (this._filter)
             filterIndex = this._user.alias.toLowerCase().indexOf(this._filter);
@@ -525,16 +520,16 @@ var UserListRow = new Lang.Class({
             let postMatch = this._user.alias.substring(filterIndex + this._filter.length);
             this._label.label = preMatch + '<b>' + theMatch + '</b>' + postMatch;
         }
-    },
+    }
 
-    _updateArrowVisibility: function() {
+    _updateArrowVisibility() {
         let flags = this.get_state_flags();
         this._arrow.visible = this.expand ||
                               flags & Gtk.StateFlags.PRELIGHT ||
                               flags & Gtk.StateFlags.FOCUSED;
-    },
+    }
 
-    _onExpandedChanged: function() {
+    _onExpandedChanged() {
         if (this._revealer.reveal_child) {
             this.get_style_context().add_class('expanded');
             this._arrow.arrow_type = Gtk.ArrowType.DOWN;
@@ -546,12 +541,10 @@ var UserListRow = new Lang.Class({
     }
 });
 
-var UserList = new Lang.Class({
-    Name: 'UserList',
-    Extends: Gtk.ScrolledWindow,
-
-    _init: function(room) {
-        this.parent({ hexpand: true,
+var UserList = GObject.registerClass(
+class UserList extends Gtk.ScrolledWindow {
+    _init(room) {
+        super._init({ hexpand: true,
                       shadow_type: Gtk.ShadowType.ETCHED_IN,
                       hscrollbar_policy: Gtk.PolicyType.NEVER,
                       propagate_natural_width: true });
@@ -575,44 +568,42 @@ var UserList = new Lang.Class({
 
         this._updateHeightId = 0;
         this._list.connect('size-allocate',
-                           Lang.bind(this, this._updateContentHeight));
+                           this._updateContentHeight.bind(this));
 
         this._list.set_selection_mode(Gtk.SelectionMode.NONE);
         /* see https://bugzilla.gnome.org/show_bug.cgi?id=725403 */
-        //this._list.set_header_func(Lang.bind(this, this._updateHeader));
+        //this._list.set_header_func(this._updateHeader.bind(this));
         this._filter = '';
-        this._list.set_filter_func(Lang.bind(this, this._filterRows));
-        this._list.set_sort_func(Lang.bind(this, this._sort));
+        this._list.set_filter_func(this._filterRows.bind(this));
+        this._list.set_sort_func(this._sort.bind(this));
 
-        this._list.connect('row-activated',
-                           Lang.bind(this, this._onRowActivated));
-        this.connect('destroy',
-                     Lang.bind(this, this._onDestroy));
+        this._list.connect('row-activated', this._onRowActivated.bind(this));
+        this.connect('destroy', this._onDestroy.bind(this));
 
         this._room = room;
-        this._rows = {};
+        this._rows = new Map();
         this._activeRow = null;
 
         let roomSignals = [
             { name: 'member-renamed',
-              handler: Lang.bind(this, this._onMemberRenamed) },
+              handler: this._onMemberRenamed.bind(this) },
             { name: 'member-disconnected',
-              handler: Lang.bind(this, this._onMemberRemoved) },
+              handler: this._onMemberRemoved.bind(this) },
             { name: 'member-kicked',
-              handler: Lang.bind(this, this._onMemberRemoved) },
+              handler: this._onMemberRemoved.bind(this) },
             { name: 'member-banned',
-              handler: Lang.bind(this, this._onMemberRemoved) },
+              handler: this._onMemberRemoved.bind(this) },
             { name: 'member-left',
-              handler: Lang.bind(this, this._onMemberRemoved) },
+              handler: this._onMemberRemoved.bind(this) },
             { name: 'member-joined',
-              handler: Lang.bind(this, this._onMemberJoined) },
+              handler: this._onMemberJoined.bind(this) },
             /*
             // see https://bugzilla.gnome.org/show_bug.cgi?id=725403
             { name: 'members-changed',
-              handler: Lang.bind(this, this._onMembersChanged) },
+              handler: this._onMembersChanged.bind(this) },
             */
             { name: 'notify::channel',
-              handler: Lang.bind(this, this._onChannelChanged) }
+              handler: this._onChannelChanged.bind(this) }
         ];
         this._roomSignals = [];
         roomSignals.forEach(signal => {
@@ -621,24 +612,24 @@ var UserList = new Lang.Class({
         this._onChannelChanged(room);
 
         this.show_all();
-    },
+    }
 
     get numRows() {
-        return Object.keys(this._rows).length;
-    },
+        return this._rows.size;
+    }
 
-    _onDestroy: function() {
+    _onDestroy() {
         for (let i = 0; i < this._roomSignals.length; i++)
             this._room.disconnect(this._roomSignals[i]);
         this._roomSignals = [];
-    },
+    }
 
-    setFilter: function(filter) {
+    setFilter(filter) {
         this._filter = filter;
         this._list.invalidate_filter();
-    },
+    }
 
-    _updateContentHeight: function() {
+    _updateContentHeight() {
         if (this._updateHeightId != 0)
             return;
 
@@ -660,28 +651,28 @@ var UserList = new Lang.Class({
             this._updateHeightId = 0;
             return GLib.SOURCE_REMOVE;
         });
-    },
+    }
 
-    _onMemberRenamed: function(room, oldMember, newMember) {
+    _onMemberRenamed(room, oldMember, newMember) {
         this._removeMember(oldMember);
         this._addMember(newMember);
-    },
+    }
 
-    _onMemberRemoved: function(room, member) {
+    _onMemberRemoved(room, member) {
         this._removeMember(member);
-    },
+    }
 
-    _onMemberJoined: function(room, member) {
+    _onMemberJoined(room, member) {
         this._addMember(member);
-    },
+    }
 
-    _onMembersChanged: function(room) {
+    _onMembersChanged(room) {
         this._counterLabel.label = this.numRows.toString();
-    },
+    }
 
-    _onChannelChanged: function(room) {
+    _onChannelChanged(room) {
         this._list.foreach(w => { w.destroy(); });
-        this._rows = {};
+        this._rows.clear();
 
         if (!room.channel)
             return;
@@ -689,42 +680,42 @@ var UserList = new Lang.Class({
         let members = room.channel.group_dup_members_contacts();
         for (let i = 0; i < members.length; i++)
             this._addMember(members[i]);
-    },
+    }
 
-    _addMember: function(member) {
+    _addMember(member) {
         let row = new UserListRow(member);
-        this._rows[member] = row;
+        this._rows.set(member, row);
         this._list.add(row);
-    },
+    }
 
-    _removeMember: function(member) {
-        let row = this._rows[member];
+    _removeMember(member) {
+        let row = this._rows.get(member);
         if (row)
             row.destroy();
-        delete this._rows[member];
-    },
+        this._rows.delete(member);
+    }
 
-    _setActiveRow: function(row) {
+    _setActiveRow(row) {
         if (this._activeRow && this._activeRow != row)
             this._activeRow.expand = false;
         this._activeRow = row;
-    },
+    }
 
-    _onRowActivated: function(list, row) {
+    _onRowActivated(list, row) {
         this._setActiveRow(row);
         this._activeRow.expand = !this._activeRow.expand;
-    },
+    }
 
-    _sort: function(row1, row2) {
+    _sort(row1, row2) {
         return row1.user.alias.localeCompare(row2.user.alias);
-    },
+    }
 
-    _filterRows: function(row) {
+    _filterRows(row) {
         row.setFilter(this._filter);
         return row.shouldShow();
-    },
+    }
 
-    _updateHeader: function(row, before) {
+    _updateHeader(row, before) {
         if (before) {
             row.set_header(null);
             return;
