@@ -1,60 +1,51 @@
-const Polari = imports.gi.Polari;
-const Lang = imports.lang;
-const Tp = imports.gi.TelepathyGLib;
-const Signals = imports.signals;
-const GObject = imports.gi.GObject;
-const Utils = imports.utils;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
+const GObject = imports.gi.GObject;
+const Polari = imports.gi.Polari;
+const Tp = imports.gi.TelepathyGLib;
 
-const AccountsMonitor = imports.accountsMonitor;
-const RoomManager = imports.roomManager;
+const {AccountsMonitor} = imports.accountsMonitor;
+const {RoomManager} = imports.roomManager;
+const Utils = imports.utils;
 
-let _singleton = null;
+var UserStatusMonitor = class {
+    static getDefault() {
+        if (!this._singleton)
+            this._singleton = new UserStatusMonitor();
+        return this._singleton;
+    }
 
-function getUserStatusMonitor() {
-    if (_singleton == null)
-        _singleton = new UserStatusMonitor();
-    return _singleton;
-}
-
-var UserStatusMonitor = new Lang.Class({
-    Name: 'UserStatusMonitor',
-
-    _init: function() {
+    constructor() {
         this._userTrackers = new Map();
         this._accountsMonitor = AccountsMonitor.getDefault();
 
         this._accountsMonitor.connect('account-added',
-                                      Lang.bind(this, this._onAccountAdded));
+                                      this._onAccountAdded.bind(this));
         this._accountsMonitor.connect('account-removed',
-                                      Lang.bind(this, this._onAccountRemoved));
+                                      this._onAccountRemoved.bind(this));
 
         this._accountsMonitor.accounts.forEach(
                     a => { this._onAccountAdded(this._accountsMonitor, a); });
-    },
+    }
 
-    _onAccountAdded: function(accountsMonitor, account) {
+    _onAccountAdded(accountsMonitor, account) {
         if (this._userTrackers.has(account))
             return;
 
         this._userTrackers.set(account, new UserTracker(account));
-    },
+    }
 
-    _onAccountRemoved: function(accountsMonitor, account) {
+    _onAccountRemoved(accountsMonitor, account) {
         this._userTrackers.delete(account);
-    },
+    }
 
-    getUserTrackerForAccount: function(account) {
+    getUserTrackerForAccount(account) {
             return this._userTrackers.get(account);
     }
-});
+};
 
 
-var UserTracker = new Lang.Class({
-    Name: 'UserTracker',
-    Extends: GObject.Object,
-
+var UserTracker = GObject.registerClass({
     Signals: {
         'status-changed': {
             flags: GObject.SignalFlags.DETAILED,
@@ -64,10 +55,10 @@ var UserTracker = new Lang.Class({
             flags: GObject.SignalFlags.DETAILED,
             param_types: [GObject.TYPE_STRING]
         }
-    },
-
-    _init: function(account) {
-        this.parent();
+    }
+}, class UserTracker extends GObject.Object {
+    _init(account) {
+        super._init();
 
         this._account = account;
 
@@ -76,31 +67,31 @@ var UserTracker = new Lang.Class({
         this._handlerCounter = 0;
         this._app = Gio.Application.get_default();
 
-        this._app.connect('prepare-shutdown', Lang.bind(this, this._onShutdown));
+        this._app.connect('prepare-shutdown', this._onShutdown.bind(this));
 
         this._roomManager = RoomManager.getDefault();
-        this._roomManager.connect('room-added', Lang.bind(this, this._onRoomAdded));
-        this._roomManager.connect('room-removed', Lang.bind(this, this._onRoomRemoved));
-    },
+        this._roomManager.connect('room-added', this._onRoomAdded.bind(this));
+        this._roomManager.connect('room-removed', this._onRoomRemoved.bind(this));
+    }
 
-    _onShutdown: function() {
+    _onShutdown() {
         for (let room of this._roomData.keys())
             this._onRoomRemoved(this._roomManager, room);
-    },
+    }
 
-    _getRoomContacts: function(room) {
+    _getRoomContacts(room) {
         return this._roomData.get(room).contactMapping;
-    },
+    }
 
-    _getRoomHandlers: function(room) {
+    _getRoomHandlers(room) {
         return this._roomData.get(room).handlerMapping;
-    },
+    }
 
-    _getRoomSignals: function(room) {
+    _getRoomSignals(room) {
         return this._roomData.get(room).roomSignals;
-    },
+    }
 
-    _onRoomAdded: function(roomManager, room) {
+    _onRoomAdded(roomManager, room) {
         if (room.account != this._account)
             return;
 
@@ -108,37 +99,37 @@ var UserTracker = new Lang.Class({
 
         let roomSignals = [
             { name: 'notify::channel',
-              handler: Lang.bind(this, this._onChannelChanged) },
+              handler: this._onChannelChanged.bind(this) },
             { name: 'member-renamed',
-              handler: Lang.bind(this, this._onMemberRenamed) },
+              handler: this._onMemberRenamed.bind(this) },
             { name: 'member-disconnected',
-              handler: Lang.bind(this, this._onMemberLeft) },
+              handler: this._onMemberLeft.bind(this) },
             { name: 'member-kicked',
-              handler: Lang.bind(this, this._onMemberLeft) },
+              handler: this._onMemberLeft.bind(this) },
             { name: 'member-banned',
-              handler: Lang.bind(this, this._onMemberLeft) },
+              handler: this._onMemberLeft.bind(this) },
             { name: 'member-joined',
-              handler: Lang.bind(this, this._onMemberJoined) },
+              handler: this._onMemberJoined.bind(this) },
             { name: 'member-left',
-              handler: Lang.bind(this, this._onMemberLeft) }
+              handler: this._onMemberLeft.bind(this) }
         ];
 
         let signalIds = this._getRoomSignals(room);
         roomSignals.forEach(signal => {
             signalIds.push(room.connect(signal.name, signal.handler));
         });
-    },
+    }
 
-    _onRoomRemoved: function(roomManager, room) {
+    _onRoomRemoved(roomManager, room) {
         if (!this._roomData.has(room))
             return;
 
         this._getRoomSignals(room).forEach(id => { room.disconnect(id); });
         this._clearUsersFromRoom(room);
         this._roomData.delete(room);
-    },
+    }
 
-    _onChannelChanged: function(room) {
+    _onChannelChanged(room) {
         if (!room.channel) {
             this._clearUsersFromRoom(room);
             return;
@@ -153,51 +144,51 @@ var UserTracker = new Lang.Class({
         /*keep track of initial members in the room, both locally and
         globally*/
         members.forEach(m => { this._trackMember(m, room); });
-    },
+    }
 
-    _clearUsersFromRoom: function(room) {
+    _clearUsersFromRoom(room) {
         let map = this._getRoomContacts(room);
         for (let [baseNick, contacts] of map)
             contacts.slice().forEach((m) => { this._untrackMember(m, room); });
-    },
+    }
 
-    _ensureRoomMappingForRoom: function(room) {
+    _ensureRoomMappingForRoom(room) {
         if (this._roomData.has(room))
             return;
         this._roomData.set(room, { contactMapping: new Map(),
                                    handlerMapping: new Map(),
                                    roomSignals: [] });
-    },
+    }
 
-    _onMemberRenamed: function(room, oldMember, newMember) {
+    _onMemberRenamed(room, oldMember, newMember) {
         this._untrackMember(oldMember, room);
         this._trackMember(newMember, room);
-    },
+    }
 
-    _onMemberJoined: function(room, member) {
+    _onMemberJoined(room, member) {
         this._trackMember(member, room);
-    },
+    }
 
-    _onMemberLeft: function(room, member) {
+    _onMemberLeft(room, member) {
         this._untrackMember(member, room);
-    },
+    }
 
-    _runHandlers: function(room, member, status) {
+    _runHandlers(room, member, status) {
         let baseNick = Polari.util_get_basenick(member.alias);
         let roomHandlers = this._getRoomHandlers(room);
         for (let [id, info] of roomHandlers)
             if (!info.nickName || info.nickName == baseNick)
                 info.handler(baseNick, status);
-    },
+    }
 
-    _pushMember: function(map, baseNick, member) {
+    _pushMember(map, baseNick, member) {
         if (!map.has(baseNick))
             map.set(baseNick, []);
         let contacts = map.get(baseNick);
         return contacts.push(member);
-    },
+    }
 
-    _trackMember: function(member, room) {
+    _trackMember(member, room) {
         let baseNick = Polari.util_get_basenick(member.alias);
         let status = Tp.ConnectionPresenceType.AVAILABLE;
 
@@ -222,18 +213,18 @@ var UserTracker = new Lang.Class({
         }
 
         this.emit("contacts-changed::" + baseNick, member.alias);
-    },
+    }
 
-    _popMember: function(map, baseNick, member) {
+    _popMember(map, baseNick, member) {
         let contacts = map.get(baseNick) || [];
         let index = contacts.map(c => c.alias).indexOf(member.alias);
         if (index < 0)
             return [false, contacts.length];
         contacts.splice(index, 1);
         return [true, contacts.length];
-    },
+    }
 
-    _untrackMember: function(member, room) {
+    _untrackMember(member, room) {
         let baseNick = Polari.util_get_basenick(member.alias);
         let status = Tp.ConnectionPresenceType.OFFLINE;
 
@@ -259,17 +250,17 @@ var UserTracker = new Lang.Class({
             }
             this.emit("contacts-changed::" + baseNick, member.alias);
         }
-    },
+    }
 
-    getNickStatus: function(nickName) {
+    getNickStatus(nickName) {
         let baseNick = Polari.util_get_basenick(nickName);
 
         let contacts = this._baseNickContacts.get(baseNick) || [];
         return contacts.length == 0 ? Tp.ConnectionPresenceType.OFFLINE
                                     : Tp.ConnectionPresenceType.AVAILABLE;
-    },
+    }
 
-    getNickRoomStatus: function(nickName, room) {
+    getNickRoomStatus(nickName, room) {
         let baseNick = Polari.util_get_basenick(nickName);
 
         this._ensureRoomMappingForRoom(room);
@@ -277,9 +268,9 @@ var UserTracker = new Lang.Class({
         let contacts = this._getRoomContacts(room).get(baseNick) || [];
         return contacts.length == 0 ? Tp.ConnectionPresenceType.OFFLINE
                                     : Tp.ConnectionPresenceType.AVAILABLE;
-    },
+    }
 
-    lookupContact: function(nickName) {
+    lookupContact(nickName) {
         let baseNick = Polari.util_get_basenick(nickName);
 
         let contacts = this._baseNickContacts.get(baseNick) || [];
@@ -291,9 +282,9 @@ var UserTracker = new Lang.Class({
                 return contacts[i];
 
         return contacts[0];
-    },
+    }
 
-    watchRoomStatus: function(room, baseNick, callback) {
+    watchRoomStatus(room, baseNick, callback) {
         this._ensureRoomMappingForRoom(room);
 
         this._getRoomHandlers(room).set(++this._handlerCounter, {
@@ -302,15 +293,15 @@ var UserTracker = new Lang.Class({
         });
 
         return this._handlerCounter;
-    },
+    }
 
-    unwatchRoomStatus: function(room, handlerID) {
+    unwatchRoomStatus(room, handlerID) {
         if (!this._roomData.has(room))
             return;
         this._getRoomHandlers(room).delete(handlerID);
-    },
+    }
 
-    _notifyNickAvailable: function (member, room) {
+    _notifyNickAvailable (member, room) {
         let notification = new Gio.Notification();
         notification.set_title(_("User is online"));
         notification.set_body(_("User %s is now online.").format(member.alias));
@@ -324,29 +315,29 @@ var UserTracker = new Lang.Class({
         this._app.send_notification(this._getNotifyActionNameInternal(member.alias), notification);
 
         let baseNick = Polari.util_get_basenick(member.alias);
-    },
+    }
 
-    _shouldNotifyNick: function(nickName) {
+    _shouldNotifyNick(nickName) {
         let actionName = this._getNotifyActionNameInternal(nickName);
         let state = this._app.get_action_state(actionName);
         return state ? state.get_boolean()
                      : false;
-    },
+    }
 
-    _setNotifyActionEnabled: function(nickName, enabled) {
+    _setNotifyActionEnabled(nickName, enabled) {
         let name = this._getNotifyActionNameInternal(nickName);
         let action = this._app.lookup_action(name);
         if (action)
             action.enabled = enabled;
-    },
+    }
 
-    _getNotifyActionNameInternal: function(nickName) {
+    _getNotifyActionNameInternal(nickName) {
         return 'notify-user-' +
                this._account.get_path_suffix() + '-' +
                Polari.util_get_basenick(nickName);
-    },
+    }
 
-    getNotifyActionName: function(nickName) {
+    getNotifyActionName(nickName) {
         let name = this._getNotifyActionNameInternal(nickName);
 
         if (!this._app.lookup_action(name)) {
